
/* Coordinates are measured in micrometers                */
/* Supported paper sizes: DIN-A4, portrait and landscape  */
#include "string.h"
#include "u_pl_loc.h"
#include "u_plot.h"
#include <stdio.h>
#include <stdlib.h>

typedef enum orients {UNKNOWN,PORTRAIT,LANDSCAPE} orients;

#define A4_X  190000
#define A4_Y  270000

#define FACTOR 10000.0       /* micrometers per centimeter */

static FILE *plotfile;
static int file_is_open, is_graphmode = FALSE;
static orients format, orient;
static int rotate, devsizex, devsizey, color;

static void write_point(char *cmd, int x, int y){
   int tmp;
   if (rotate) { tmp = x; x = devsizey - y; y = tmp; }
   fprintf(plotfile," %g %g %s\n",x/FACTOR,y/FACTOR,cmd);
}

static int plotd_modify_ps(int nh, handletyp *h, int sizex, int sizey,
  		       int devtype, int paper, char name[]){
  switch (paper){
    case PLOT_PAPER_A4P_AUTO : 
      format = PORTRAIT;
      rotate = FALSE;
      devsizex = A4_X;
      devsizey = A4_Y; 
      fprintf(plotfile,"%%%% Paper format DIN A4 Portrait\n"); break;
    case PLOT_PAPER_A4L_AUTO : 
      format = LANDSCAPE;  
      rotate = TRUE;
      devsizex = A4_Y;
      devsizey = A4_X;
      fprintf(plotfile,"%%%% Paper format DIN A4 Landscape on Laserjet 4M\n"); 
      fprintf(plotfile,"%%%% All coordinates rotated by 90 degrees\n"); break;
    case PLOT_PAPER_FREE :
      format = UNKNOWN;
      rotate = FALSE;
      devsizex = sizex;
      devsizey = sizey; 
      fprintf(plotfile,"   Free paper format: %4.2f x %4.2f cm\n",
              devsizex/FACTOR,devsizey/FACTOR);
      break;
  }
  return PLOT_STAT_OK;
}

static void plotd_settextalign_ps(int handle, int xalign, int yalign){}

static int plotd_getdevice_ps(int handle, int *sizex, int *sizey){
   *sizex = devsizex;   *sizey = devsizey;
   return PLOT_STAT_OK;
}

static void plotd_line_ps(int handle, int X0, int Y0, int X1, int Y1){
    printf("plotd_line_ps   write_point");
   write_point("MT",X0,Y0);
   write_point("DT\nST",X1,Y1);
}

static void plotd_point_ps(int handle, int X, int Y){
   plotd_line_ps(handle,X,Y,X,Y);
}

static void plotd_polyline_ps(int handle, int pts, int X[], int Y[]){
  int i;     
  write_point("MT",X[0],Y[0]);
  for (i=1; i<pts; i++) write_point("DT",X[i],Y[i]);
  fprintf(plotfile,"ST\n");
}

void plotd_fillpolygon_ps(int handle,int pts,int X[], int Y[]){
  int i;    
  write_point("MT",X[0],Y[0]);
  for (i=1; i<pts; i++) write_point("DT",X[i],Y[i]);
  fprintf(plotfile,"fill\n");
  fprintf(plotfile,"ST\n");
}

static int plotd_setfg_ps(int handle, int fg){ 
    /*fg = fg % 16;*/
    /*if (fg == 0) fg = 1; spiering*/
  fprintf(plotfile,"SP%d\n",fg);
  color = fg; 
  /*printf("  fg=%d \n",fg);*/
  return color; 
}

static int plotd_getfg_ps(int handle){ return color;}

static int plotd_setbg_ps(int handle, int bg){ return 0 ;}

static int plotd_getbg_ps(int handle){ return 0;}

static void plotd_writetext_ps(int handle, int x, int y, char txt[]){
   int tmp;
   fprintf(plotfile,"  %g  %g  %g DI\n",0.3,0.5,0.0);
   if (rotate){
     tmp = x; x = devsizex - y; y = tmp;
     fprintf(plotfile,"  %g  %g  %g  (%s) LB\n",x/FACTOR,y/FACTOR,90.0,txt);
   }else
     fprintf(plotfile,"  %g  %g  %g  (%s) LB\n",x/FACTOR,y/FACTOR,0.0,txt);
}

static void plotd_erasetext_ps(int handle, int x, int y, char txt[]){}

static void plotd_graphmode_ps(int handle){
   if (is_graphmode) return;
   fprintf(plotfile,"gsave\n");
   is_graphmode = TRUE;
}

static void plotd_clear_ps(int handle){
}

static void plotd_textmode_ps(int handle){
   if (! is_graphmode) return;
   fprintf(plotfile,"grestore showpage\n");
   is_graphmode = FALSE;
}

static void plotd_close_ps(int handle){
   if (is_graphmode) fprintf(plotfile,"grestore showpage\n");
   fclose(plotfile);
   file_is_open = FALSE;
   is_graphmode = FALSE;
}

static int plotd_setmode_ps(int handle, int mode){ return 0;}

static int plotd_getmode_ps(int handle){ return 0; }

static void plotd_do_special_ps(int handle, char str[]){
   if (! file_is_open) return;
   fprintf(plotfile,"%s\n",str);
}

int plotd_init_ps(int nh, handletyp *h, int sizex, int sizey,
		     int posx, int posy, char name[], void *special){
   FILE *headerfile;
   char *temp, headerfilename[128];
   char tmp[81];
   int *bbox;

   strcpy(headerfilename,"PLOT_HEADER.PS");
   /*printf("headerfilename=%s \n",headerfilename);*/
   /*temp=getenv("PSPLOT_HEADER");
     if (temp) strcpy(headerfilename,temp); spiering*/

   if (file_is_open) return PLOT_STAT_NOMOREDEV;
   headerfile = fopen(headerfilename,"r"); 
   if (! headerfile) {
    printf("PSPLOT_HEADER file not copied to working directory \n");
    printf("-> .ps file cannot be printed correctly\n");
    return PLOT_STAT_NODRIVER;
   }
   plotfile = fopen(name,"w");
   if (! plotfile) { fclose(headerfile); return(PLOT_STAT_OPENERROR); }
   
   file_is_open = TRUE;
   is_graphmode = TRUE;
   color = 1;
   fprintf(plotfile,"%%!PS\n");
   if (special != NULL){
     bbox = (int *)special;
     fprintf(plotfile, "%%%%BoundingBox: %d %d %d %d\n", 
             bbox[0],bbox[1],bbox[2],bbox[3]);
   }
  
   while (fgets(tmp,80,headerfile)) fputs(tmp,plotfile);
   h->ploth_modify     =   plotd_modify_ps;
   h->ploth_getdevice  =   plotd_getdevice_ps;
   h->ploth_line       =   plotd_line_ps;
   h->ploth_point      =   plotd_point_ps;
   h->ploth_polyline   =   plotd_polyline_ps;
   h->ploth_fillpolygon=   plotd_fillpolygon_ps;
   h->ploth_setfg      =   plotd_setfg_ps;
   h->ploth_getfg      =   plotd_getfg_ps;
   h->ploth_setbg      =   plotd_setbg_ps;
   h->ploth_getbg      =   plotd_getbg_ps;
   h->ploth_writetext  =   plotd_writetext_ps;
   h->ploth_erasetext  =   plotd_erasetext_ps;
   h->ploth_graphmode  =   plotd_graphmode_ps;
   h->ploth_clear      =   plotd_clear_ps;
   h->ploth_textmode   =   plotd_textmode_ps;
   h->ploth_close      =   plotd_close_ps;
   h->ploth_setmode    =   plotd_setmode_ps;
   h->ploth_getmode    =   plotd_getmode_ps;
   h->ploth_do_special =   plotd_do_special_ps;
   h->capabilities     = PLOT_CAPABILITY_HARDCOPY 
                       + PLOT_CAPABILITY_CM;
   h->cm_per_unit      = 0.0001;
   plotd_modify_ps(nh,h,sizex,sizey,posx,posy,name);
   return(PLOT_STAT_OK);
}
