#include "v_inp.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <X11/cursorfont.h>
#include <X11/Xutil.h>   /*includes Xlib.h*/

#include "u_pl_x11.h"
#include "u_pl_loc.h"
#include "u_plot.h"

#include <X11/Xlib.h>
#include <X11/cursorfont.h>
#include <X11/Xutil.h>

#define FontName1 "-ADOBE-NEW CENTURY SCHOOLBOOK-MEDIUM-R-NORMAL--*-140-*-*-P-*"
#define FontName2 "-urw-century shoolbook l-medium-r-normal--*-140-*-*-p-*"
#define WindowName "Graphic Display"

#define COLENV "COLORFILE"
#define COLFILE "x11_ps_colors.txt"

#define DEBUG FALSE

/* Default colors, used if color file not found*/
/* read from /usr/share/emacs/21.3/etc/rgb.txt*/

#define MAXCOLOR  16
   static char fixcolors[MAXCOLOR][30] = {
           "black", 
           "white",
           "red",
           "green",
           "blue",
           "yellow",
           "magenta",
           "cyan",
           "orange",
           "gold",
           "brown",
           "violet",
           "purple",
           "lavender",
           "Cornsilk",
           "Ivory"       /* 16 */};
static char colors[99][30];
static int mcolorsff;
/* We use arrays instead of dynamic allocation; on a XWindow-System */
/* some KB do not matter                                            */

static Display *display = 0;
static int input_active[MAXHANDLES];
static Window windows[MAXHANDLES];                        /* Windows         */ 
static GC gc[MAXHANDLES];                                 /* Graphic Context */
static Screen *screen = NULL;                                    /* Screen          */
static unsigned int dispsizex, dispsizey;                 /* Size of Screen  */
static int devsizex[MAXHANDLES], devsizey[MAXHANDLES];    /* Size of Window  */
static int foreground[MAXHANDLES], background[MAXHANDLES];/* Colours         */ 
static int text_xalign[MAXHANDLES],text_yalign[MAXHANDLES];/*Text alignment  */

/* variables for save of input focus */
static Window save_focus[MAXHANDLES];                      
static int save_revert[MAXHANDLES], is_graphmode[MAXHANDLES];

/* Default properties of windows */
#define DEFWIN_W    400
#define DEFWIN_H    300
#define DEFWIN_X     10
#define DEFWIN_Y     10
#define DEFBORDER     5
#define DEFFOREGR     0
#define DEFBACKGR     1

extern char  FontName[5][80];

int plotd_sel_font(int nh,int sizex){
   static Font font;
   XFontStruct *xf;
   char *Fname;
   int i,j,ii,ix;

   j=1;
   ii=sizex;
   nofont:
   if(ii>0){
    ix=sizex/150;
    i=ix;
    ii=0;
    if(i<2) i=2; if(i>6)i=6;
    ix=i; 
   }else{i=j;}
    xf=XLoadQueryFont(display,FontName[i-2]);
    if(xf==NULL)printf("*%d %s \n",i-1,FontName[i-2]);
    if(xf!=NULL) font = XLoadFont(display,FontName[i-2]); 
    j=j+1;
    /*printf("j=%d i=%d  %s \n",j,i-2,FontName[i-2]);*/
   if(xf == NULL && j<7) goto nofont;
   if(i != ix) {
     printf("\n fonts listed in PLOT.FONT are not implemented (*)\n");
     printf(" take from the list printed by the command:xlsfonts  \n");}
     if(i != ix && j<7) 
     printf(" size of font number %d may not fit to the size of plot\n",i-1);
   if(xf!=NULL) {j=PLOT_STAT_OK;
      if(nh==0) XSetFont(display, gc[nh], font);
   }else{j=12;}
   return(j);
}
/*************************** Local routines ***************************/
/* Remove all events pending for this window, we don't want to redraw */
/* more than once                                                     */
static void RemovePendingEvents(int handle){
    XEvent event;
    do{} while(XCheckWindowEvent(display, windows[handle],
			      ButtonPressMask | KeyPressMask | ExposureMask,
			      &event));
}


static int doButtonPress(XButtonEvent *ep, int *x, int *y)
/* Handle Mouse Button and Keyboard Events */
{   char buffer;
    int nchars;
    /* Get character code from event */
/*    nchars = XLookupString((XKeyEvent*)&ep,&buffer,10,NULL,NULL);*/
    *x = ep->x; *y = ep->y;
    switch (ep->button){ /* Mouse buttons 1,2,3 */
       case Button1: return(PLOT_TRM_MB_LEFT); break;
       case Button2: return(PLOT_TRM_MB_CENTER); break;
       case Button3: return(PLOT_TRM_MB_RIGHT); break;
       /* No mouse button */
/*       default : if (nchars==1) return buffer;*/ /* normal key --> char */
/*                 return 0;                   */  /* other key  -->    0 */
    }
}

int doDefineColor(int n){
/* Allocate a color by name                       */
/* For color names, see e.g. /usr/lib/X11/rgb.txt */
   int pixel;
   XColor exact_color,screen_color;
   if ((n < 0) || (n > mcolorsff)){ 
     if (DEBUG) {printf(" Colour out of range: %d",n);newline();};
      return XBlackPixelOfScreen(screen); }
   if ((XDefaultVisualOfScreen(screen))->class == TrueColor
       || (XDefaultVisualOfScreen(screen))->class == PseudoColor
       || (XDefaultVisualOfScreen(screen))->class == DirectColor
       || (XDefaultVisualOfScreen(screen))->class == StaticColor){
       if (XAllocNamedColor(display, XDefaultColormapOfScreen(screen),
	    colors[n], &screen_color, &exact_color)){
	 if (DEBUG) {printf("n=%d Named color allocated: %s",n,colors[n]);
             printf(" red: %d  green: %d  blue: %d",
            exact_color.red,exact_color.green,screen_color.blue);newline();};
           return screen_color.pixel;}
       else {printf("n=%d  Named color failed: <<%s>>",n,colors[n]);newline();};
   }
   else  if (DEBUG) {printf("No colour system");newline();};
   if (n == 1){
     if (DEBUG) {printf(" Returning White Pixel");newline();};
      return XWhitePixelOfScreen(screen);}
   if (DEBUG) {printf(" Returning Black Pixel");newline();};
   return XBlackPixelOfScreen(screen);
}


/**************** End of Local routines **************/

int plotd_modify_X11(int nh,/* handletyp *h,modif. GA:31.05.1995*/ int sizex, int sizey,
  		     int posx, int posy, char name[]){
/* Change window size, position or title */ 
    XSizeHints xsh;

    if (name) XStoreName(display, windows[nh], name);
    xsh.flags = 0;
    if ((sizex > 0) && (sizey > 0)){
         xsh.width = sizex;   xsh.height = sizey;
         xsh.flags = xsh.flags | PSize; }
    if ((posx > 0) && (posy > 0)){
	 xsh.x = posx;    xsh.y = posy;
         xsh.flags = xsh.flags | PPosition; }
    if (xsh.flags) XSetNormalHints(display, windows[nh], &xsh);
    return 1;
}

int plotd_getdevice_X11(int handle, int *sizex, int *sizey){
/* get actual size of window */
   unsigned udummy;
   int dummy;
   Window rootwin;
   XGetGeometry(display,windows[handle],&rootwin,&dummy,&dummy,
        	(unsigned*)sizex,(unsigned*)sizey,&udummy,&udummy);
   devsizex[handle] = *sizex;
   devsizey[handle] = *sizey;
   return(PLOT_DEV_X11);
}


void plotd_line_X11(int handle, int X0, int Y0, int X1, int Y1){
/* Draw a line */
   XDrawLine(display,windows[handle],gc[handle],X0,devsizey[handle]-Y0,
	                                    X1,devsizey[handle]-Y1);
}

void plotd_point_X11(int handle, int X, int Y){
/* Plot a point, i.e. a line of length 0 */ 
   plotd_line_X11(handle,X,Y,X,Y);
}

void plotd_polyline_X11(int handle, int pts, int X[], int Y[]){
/* Draw a polyline */
     short int i;
     XPoint *itmp;
     itmp = malloc(2*pts*sizeof(short int));
     for (i=0; i<pts;i++){
	 itmp[i].x=X[i]; itmp[i].y=devsizey[handle]-Y[i]; }
     XDrawLines(display,windows[handle],gc[handle],itmp,pts,CoordModeOrigin);
     free(itmp);
}
void plotd_fillpolygon_X11(int handle, int pts, int X[], int Y[]){
     short int i;
     XPoint *itmp;
     itmp = malloc(2*pts*sizeof(short int));
     for (i=0; i<pts;i++){
	 itmp[i].x=X[i]; itmp[i].y=devsizey[handle]-Y[i]; }
/* fill polygon with foreground color */
XFillPolygon(display,windows[handle],gc[handle],itmp,pts,Convex,CoordModeOrigin);
}

int plotd_setfg_X11(int handle, int fg){
/* set foreground color */ 
    /*printf(" in  fg=%d ",fg);*/
    XGCValues xgcv;
    if (fg==PLOT_USEDEFAULT) fg = DEFFOREGR;
    fg = fg % (mcolorsff+1);
    /*if (fg==background[handle]) fg = DEFFOREGR;  spiering*/
    foreground[handle] = fg;
     /*printf("  out fg=%d \n",fg);*/
    XSetForeground(display,gc[handle],doDefineColor(fg));
    return(foreground[handle]);
}

int plotd_getfg_X11(int handle){
/* get foreground color */
   return(foreground[handle]);
}

int plotd_setbg_X11(int handle, int bg){
/* set background color */
    /*printf(" in  bg=%d ",bg);*/
    XGCValues xgcv;
    if (bg==PLOT_USEDEFAULT) bg = DEFBACKGR;
    bg = bg % (mcolorsff+1);
    xgcv.background = doDefineColor(bg);
    background[handle] = bg; 
   /*printf("  out bg=%d \n",bg);*/
    XChangeGC(display, gc[handle], GCBackground, &xgcv);
    return(background[handle]);
}

int plotd_getbg_X11(int handle){
/* get background color */
    return(background[handle]);
}

/* On colour X-window systems, only REPLACE (= GXcopy) can be used, */
/* since colours are indices in the colour table                    */
int plotd_setmode_X11(int handle, int newmode){
   return PLOT_MODE_REPLACE;
}    

int plotd_getmode_X11(int handle){
   return PLOT_MODE_REPLACE;
}

static void TextExtent(int handle, char txt[], int *sizex, 
                       int *ascent,int *descent){
    
/* Get text extent from server: length, ascent, descent */ 
    int dirret, asc, desc;
    XCharStruct overall;
    XGCValues gcval;
    unsigned int mask;

    mask=GCFont;
    XGetGCValues(display,gc[handle],mask,&gcval);
    XQueryTextExtents(display,gcval.font,txt,strlen(txt),
                      &dirret,&asc,&desc,&overall);
    *sizex = overall.width;
    *ascent = overall.ascent;
    *descent = overall.descent;
}

void plotd_writetext_X11(int handle, int x, int y, char txt[]){
/* Text output at (x,y) using actual alignment */
    int sizex, sizey, desc, asc;
    TextExtent(handle,txt,&sizex,&asc,&desc);
    sizey = asc + desc;
    if (text_xalign[handle]==PLOT_RIGHTTEXT)  x = x - sizex;
    if (text_xalign[handle]==PLOT_CENTERTEXT) x = x - sizex/2;
    if (text_yalign[handle]==PLOT_CENTERTEXT) y = y + desc - sizey/2;
    if (text_yalign[handle]==PLOT_TOPTEXT)    y = y - asc;
    if (text_yalign[handle]==PLOT_BOTTOMTEXT) y = y + desc;
    XDrawString(display, windows[handle], gc[handle],
		x, devsizey[handle]-y, txt, strlen(txt));
}

void plotd_erasetext_X11(int handle, int x, int y, char txt[]){
/* erase text at (x,y) using actual alignment. This can only be done */
/* via XClearArea.                                                   */
    int sizex, sizey, desc, asc;
    TextExtent(handle,txt,&sizex,&asc,&desc);
    sizey = asc + desc;
    if (text_xalign[handle]==PLOT_RIGHTTEXT)  x = x - sizex;
    if (text_xalign[handle]==PLOT_CENTERTEXT) x = x - sizex/2;
    if (text_yalign[handle]==PLOT_CENTERTEXT) y = y + desc - sizey/2;
    if (text_yalign[handle]==PLOT_TOPTEXT)    y = y - asc;
    if (text_yalign[handle]==PLOT_BOTTOMTEXT) y = y + desc;
    XClearArea(display, windows[handle], x, devsizey[handle]-y-asc,
               sizex, sizey, FALSE);
}

void plotd_graphmode_X11(int handle){
/* Switch to graphic mode: bring graphic window to top, save input focus, */
/* give input focus to graphic window, and remove (expose-) events; we    */
/* know that we want to redraw.                                           */
    if (is_graphmode[handle]) return;
    XRaiseWindow(display,windows[handle]);
    XGetInputFocus(display,&save_focus[handle],&save_revert[handle]);
    XSetInputFocus(display,windows[handle],RevertToParent,CurrentTime);
    RemovePendingEvents(handle);
    text_xalign[handle] = PLOT_LEFTTEXT;
    text_yalign[handle] = PLOT_BOTTOMTEXT;
    is_graphmode[handle] = TRUE;
}

void plotd_clear_X11(int handle){
/* Clear graphic window */
    XClearWindow(display, windows[handle]);
}

void plotd_textmode_X11(int handle){
/* Switch to textmode: Lower graphic window, return input focus to terminal */
    if (! is_graphmode[handle]) return;
    XLowerWindow(display,windows[handle]);
    XSetInputFocus(display,save_focus[handle],save_revert[handle],CurrentTime);
    is_graphmode[handle] = FALSE;
}

void plotd_close_X11(int handle){
/* Destroy a graphic window */
    XFreeGC(display,gc[handle]);
    XUnmapWindow(display,windows[handle]);
    XDestroyWindow(display,windows[handle]);
}

int plotd_getpointer_X11(int handle, int *x, int *y){
/* Wait for event, return pointer position and terminator */
/* Terminator is: - key for keypress event                */
/*                - Mouse button for button event         */
/*                - PLOT_TRM_REPLOT for expose event      */
/* position is only valid for keypress and button events  */ 
   XEvent event;
   int trm;
   char buffer;
   int nchars;

   if (input_active[handle]) return PLOT_TRM_ALREADY_ACTIVE;
   input_active[handle] = TRUE;
   trm = 0;
   do { 
      XNextEvent(display, &event);
      if (event.xany.window != windows[handle]){
	 continue;
      }
      switch (event.type) {
	 case Expose: RemovePendingEvents(handle);
                      input_active[handle] = FALSE;
	              return(PLOT_TRM_REPLOT);
	 case KeyPress   :
		nchars = XLookupString(&(event.xkey),&buffer,sizeof(buffer),
					NULL,NULL);
		trm = (nchars==1) ? buffer : 0;
		/*printf("buffer:%c",buffer);newline();*/
		*x=event.xkey.x;
		*y=event.xkey.y;
		break;
         case ButtonPress:
	   /*printf("Button:%d",event.xbutton.button);newline();*/
		trm = doButtonPress(&(event.xbutton),x,y);
		break;
  	 default : ;
   }} while(trm == 0);
   *y = devsizey[handle] - *y;
   input_active[handle] = FALSE;
   return(trm);
}

int plotd_checkevent_X11(int handle, int *x, int *y){
/* Check for event, return pointer position and terminator */
/* Terminator is: - key for keypress event                 */
/*                - Mouse button for button event          */
/*                - PLOT_TRM_REPLOT for expose event       */
/*                - PLOT_TRM_ABORT if no event present     */
/* position is only valid for keypress and button events   */ 
   XEvent event;
   int trm;
   trm = 0;
   if (XCheckWindowEvent(display, windows[handle],
		     ButtonPressMask | KeyPressMask | ExposureMask,
		     &event)){
      switch (event.type) {
	 case Expose: RemovePendingEvents(handle);
	              return(PLOT_TRM_REPLOT);
	 case KeyPress   :
         case ButtonPress: trm = doButtonPress(&(event.xbutton),x,y); break;
  	 default : ;
      }
      *y = devsizey[handle] - *y;
      return(trm);}
   return PLOT_TRM_ABORT;
}

static void plotd_settextalign_X11(int handle, int xalign, int yalign){
/* set text alignment */
   text_xalign[handle] = xalign;
   text_yalign[handle] = yalign;
}

static void ReadColorNames(){
/* Read color names from file */
   XColor exact_color,screen_color;
   real rgb[99][3];
   int i;
   char *p;
   FILE *cf, *cw, *cr;
   char fname[80],headerfilename[128],temp[80], *tmp;


   if ((tmp=getenv(COLENV))!=NULL) strcpy(fname,tmp);
   else strcpy(fname,COLFILE);
   //	printf("COLFILE=%s\n",COLFILE);   
   if (!(cf=fopen(fname,"r"))){
    for (i=0; i<MAXCOLOR; i++){
        strcpy(colors[i],fixcolors[i]);
      	printf("i=%d colors[i]=%s\n",i,colors[i]);
    }
        mcolorsff=MAXCOLOR-1;

        tmp=getenv("PSPLOT_HEADER");
        if (tmp) strcpy(headerfilename,tmp); 
        cf=fopen(headerfilename,"r");

     
        if(!cf){
         printf("standard PSPLOT_HEADER file not found- PATH!\n");
         return;}
         if(fopen("PLOT_HEADER.PS","r"))return;
         cw=fopen("PLOT_HEADER.PS","w");
         fgets(temp,80,cf);
         while(strncmp(temp,"/SP4 ",5)){
         fgets(temp,80,cf); 
         fprintf(cw,"%s",temp);}
        
	for(i=5;i<mcolorsff+1;i++){
         XAllocNamedColor(display, XDefaultColormapOfScreen(screen),
			  colors[i], &screen_color, &exact_color);
	 rgb[i][1]=exact_color.red/65535.0;
	 rgb[i][2]=exact_color.green/65535.0;
	 rgb[i][3]=exact_color.blue/65535.0;
         } 
         for(i=5;i<mcolorsff+1;i++)
         fprintf(cw,
         "/SP%d { 0.03 cm setlinewidth%6.3f%6.3f%6.3f setrgbcolor} def %c %s\n",
	   i,rgb[i][1], rgb[i][2],rgb[i][3],37,colors[i]);
         /*printf(
         "/SP%d { 0.03 cm setlinewidth%6.3f%6.3f%6.3f setrgbcolor} def %c %s\n",
	 i,rgb[i][1], rgb[i][2],rgb[i][3],37,colors[i]);*/
         fclose(cf);   fclose(cw);
      return;}

     for (i=0; i<5; i++) strcpy(colors[i],fixcolors[i]);
     i=5;while(fgets(colors[i],30,cf)){
 	         p = strchr(colors[i],10);
		 if (p != 0) *p = 0;
         XAllocNamedColor(display, XDefaultColormapOfScreen(screen),
			  colors[i], &screen_color, &exact_color);
	 rgb[i][1]=exact_color.red/65535.0;
	 rgb[i][2]=exact_color.green/65535.0;
	 rgb[i][3]=exact_color.blue/65535.0;
          i++;} 
      mcolorsff=i-1;
      // printf("\n mcolorsff=%d\n ",mcolorsff);
    fclose(cf);

   tmp=getenv("PSPLOT_HEADER");
   if (tmp){ strcpy(headerfilename,tmp); 
   cf=fopen(headerfilename,"r+");
   if(fopen("PLOT_HEADER.PS","r"))return;
   cw=fopen("PLOT_HEADER.PS","w");

        fgets(temp,80,cf);
        while(strncmp(temp,"/SP4 ",5)){
        fgets(temp,80,cf); 
        fprintf(cw,"%s",temp);}
	//        fgets(temp,80,cf);
	
   for(i=5;i<mcolorsff+1;i++)
   fprintf(cw,
   "/SP%d {0.03 cm setlinewidth%6.3f%6.3f%6.3f setrgbcolor} def %c %s \n",
	   i,rgb[i][1], rgb[i][2],rgb[i][3],37,colors[i]);
      fprintf(cw,"%s\n",temp);
   fclose(cf);fclose(cw);
   //for (i=0;i<mcolorsff+1;i++)printf("ende i=%d colors[i]=%s\n",i,colors[i]);
   }
}
      
void plotd_polysymbol_X11(int handle, int pts, symboltyp sym,int x[], int y[]){
   int i,k, nseg;
   XSegment *seg;
   nseg = pts * sym.vec;			     
   seg = malloc(nseg*sizeof(XSegment));			     
   for (i=0; i<pts; i++)
     for(k=0; k<sym.vec; k++){				  
       seg[sym.vec*i+k].x1 = x[i]+sym.x1[k];			     
       seg[sym.vec*i+k].x2 = x[i]+sym.x2[k];			     
       seg[sym.vec*i+k].y1 = devsizey[handle]-(y[i]+sym.y1[k]);	     
       seg[sym.vec*i+k].y2 = devsizey[handle]-(y[i]+sym.y2[k]);
     }
   XDrawSegments(display,windows[handle],gc[handle],seg,nseg);
   free(seg);			     		     
}
			     
int plotd_init_x11(int nh, handletyp *h, int sizex, int sizey,
		     int posx, int posy, char name[]){
/* Create a plotwindow, initialize X-Window graphic if it is the first one */
   XGCValues xgcv;
   XEvent event;
   XSizeHints xsh;
   XSetWindowAttributes xswa;
   static Font font;
   unsigned udummy;
   int dummy, xdummy, ydummy;
   Window dummywin;
   Cursor the_cursor;

   /* If no display defined, create Display */
   if (display==NULL){
      display = XOpenDisplay(0);
   }
   if (display==NULL) return(PLOT_STAT_NODISPLAY);
   if (screen == NULL){    			
      screen = XDefaultScreenOfDisplay(display);
      XGetGeometry(display,XRootWindowOfScreen(screen),&dummywin,&dummy,&dummy,
        	       &dispsizex,&dispsizey,&udummy,&udummy);
      XSynchronize(display, 1);
      
      ReadColorNames();
   }
   /* Save input focus, so we can give it back to terminal */
   XGetInputFocus(display,&save_focus[nh],&save_revert[nh]);
   xswa.event_mask = ExposureMask | ButtonPressMask | KeyPressMask;
   xswa.background_pixel = doDefineColor(1);
   xswa.backing_store    = WhenMapped;
   windows[nh] = XCreateWindow(display, XRootWindowOfScreen(screen),
		 posx, posy, dispsizex, dispsizey, DEFBORDER,
		 XDefaultDepthOfScreen(screen), InputOutput,
		 XDefaultVisualOfScreen(screen),
		 CWEventMask | CWBackPixel | CWBackingStore, &xswa);
   the_cursor = XCreateFontCursor(display,XC_crosshair);
   XDefineCursor(display,windows[nh],the_cursor);
   XResizeWindow(display,windows[nh], sizex, sizey);

   xgcv.foreground = BlackPixel(display,DefaultScreen(display));
   xgcv.background = WhitePixel(display,DefaultScreen(display));
   xgcv.function = GXcopy;      
   xgcv.plane_mask = AllPlanes; 
   /*printf("XCreateGC gc[nh]  nh=%d  \n",nh);*/  
 
   gc[nh] = XCreateGC(display, windows[nh],
		  GCFunction | GCForeground | GCBackground | GCPlaneMask, 
		  &xgcv);

   dummy=plotd_sel_font(nh,sizex);
   if(dummy>0) return(12);
   xsh.x = posx;
   xsh.y = posy;
   xsh.width  = sizex;
   xsh.height = sizey;
   xsh.flags = PPosition | PSize;
   XSetNormalHints(display, windows[nh], &xsh);
   XStoreName(display, windows[nh], name);
   XMapWindow(display, windows[nh]);
   XWindowEvent(display,windows[nh], ExposureMask, &event);
   RemovePendingEvents(nh);
   plotd_getdevice_X11(nh,&xdummy,&ydummy);

   h->ploth_modify     =   plotd_modify_X11;
   h->ploth_getdevice  =   plotd_getdevice_X11;
   h->ploth_line       =   plotd_line_X11;
   h->ploth_point      =   plotd_point_X11;
   h->ploth_polyline   =   plotd_polyline_X11;
   h->ploth_setfg      =   plotd_setfg_X11;
   h->ploth_getfg      =   plotd_getfg_X11;
   h->ploth_setbg      =   plotd_setbg_X11;
   h->ploth_getbg      =   plotd_getbg_X11;
   h->ploth_writetext  =   plotd_writetext_X11;
   h->ploth_erasetext  =   plotd_erasetext_X11;
   h->ploth_graphmode  =   plotd_graphmode_X11;
   h->ploth_clear      =   plotd_clear_X11;
   h->ploth_textmode   =   plotd_textmode_X11;
   h->ploth_close      =   plotd_close_X11;
   h->ploth_setmode    =   plotd_setmode_X11;
   h->ploth_getmode    =   plotd_getmode_X11;
   h->ploth_getpointer =   plotd_getpointer_X11;
   h->ploth_checkevent =   plotd_checkevent_X11;
   h->ploth_settextalign = plotd_settextalign_X11;
   h->ploth_polysymbol   = plotd_polysymbol_X11;
   h->ploth_fillpolygon  = plotd_fillpolygon_X11;
   h->capabilities = PLOT_CAPABILITY_POINTER + PLOT_CAPABILITY_EVENT 
		   + PLOT_CAPABILITY_WINDOWS + PLOT_CAPABILITY_TEXTEXT;
   text_xalign[nh] = PLOT_LEFTTEXT;
   text_yalign[nh] = PLOT_BOTTOMTEXT;
   input_active[nh] = FALSE;		
   is_graphmode[nh] = TRUE;
   plotd_textmode_X11(nh);
   return(PLOT_STAT_OK);
}
