
#include "v_inp.h"
#include "u_plot.h"
#include "xben.h"
#include "xb_comb.h"
#include "xb_buff.h"
#include "xb_util.h"
#include "xb_menu.h"
#include <curses.h>
#include <ctype.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef PI
#define PI 3.1412
#endif

static void coord_pair(plotinfo *plot, datatype coord[], double out[]){
   double phi;
   phi = plot->angle / 180 * PI;;
   out[0] = (double)coord[0] * cos(phi) - (double)coord[1] * sin(phi); 
   out[1] = (double)coord[1] * cos(phi) + (double)coord[0] * sin(phi); 
   out[0] = out[0] * plot->factor_x + plot->shift_x;
   out[1] = out[1] * plot->factor_y + plot->shift_y;
}

static int write_points(plotinfo *srcplot, plotinfo *targplot, 
                        int npts, datatype coord[]){
   double out[2];
   int i, stat;
   coord_pair(srcplot, coord, out);
   stat = add_item(targplot, XB_ITEM_MOVE_ABS, out);
   for (i=1; i<npts; i++){
     if (stat != XB_STAT_OK){ printf(" WRITE: %d",stat);newline(); return stat;}
     coord_pair(srcplot, &coord[2*i], out);
     stat = add_item(targplot, XB_ITEM_DRAW_ABS, out);
   } 
   return stat;
}

static void combine_the_buffers(FILE *volterm,int targ, int nsrc, int src[],
                                plotinfo plot[]){
   int item, nargs, i, nplot, stat;
   datatype xmin,xmax,ymin,ymax, xdim, ydim, *argptr;
   double dummy[2];
   char outstr[80], tmp[5];
   for (i=0; i<nsrc; i++){
     if (src[i]==targ){
      XBEN_message(volterm," Error: Sources and Target must be different!"); return;
     } 
   }
   if (plot[targ].bufnum != XB_STAT_NOBUFF) delete_buffer(&plot[targ]);
   if (create_buffer(&plot[targ]) == XB_STAT_NOBUFF){
     XBEN_message(volterm," Error: No more buffers\n");
      return; }
   for (i=0; i<nsrc; i++){
      nplot = src[i];
      if (plot[nplot].bufnum == XB_STAT_NOBUFF) continue;
      stat = start_buffer(&plot[nplot]);
      if (stat != XB_STAT_OK){ printf(">>>%d",stat);newline(); continue;}
      while (stat=get_item(&plot[nplot],&item,&nargs,&argptr) != XB_STAT_END){
        switch (item){
          case XB_ITEM_NONE : break;
          case XB_ITEM_PEN : 
            dummy[0] = (double)nargs;
            stat = add_item(&plot[targ],item,dummy); break;
          case XB_ITEM_POLYLINE : 
            stat = write_points(&plot[nplot],&plot[targ],nargs,argptr); break;
          default : ;
        }
        if (stat != XB_STAT_OK) break;
      }
   }
   add_item(&plot[targ],XB_ITEM_END,dummy);
   strcpy(outstr," Combination of buffers ");
   for (i=0; i<nsrc; i++){
     sprintf(tmp," %1d",src[i]);
     strcat(outstr,tmp);
   }
   strcpy(plot[targ].filename,outstr);
   if (entscheidung(volterm," Set Visibilities of Sources \"NO\", Target \"YES\"",'Y')){
     plot[targ].visible = TRUE;
     for (i=0; i<nsrc; i++) plot[src[i]].visible = FALSE;
   }
}

void combine_buffers(FILE *volterm,plotinfo plots[]){
   int nsrc, src[16], targ, i, dummy;
   double out[16];
   char instr[80];
   dummy = -1;
   do{
     /*list_buffers(plots, &dummy);*/
     printf(" Enter source buffers : "); 
     getstr(instr);
     nsrc = trans(instr,out); if (nsrc==0) return;
     for (i=0; i<nsrc; i++) src[i] = (int)(out[i]);
     printf(" Enter target buffer  : "); 
     targ = integer_lesen(volterm);
     printf(" Combining buffers ");
     for (i=0; i<nsrc; i++) printf(" %d",src[i]);
     printf(" to buffer %d",targ);newline();
   } while (! entscheidung(volterm," Is this correct",'Y'));
   combine_the_buffers(volterm,targ, nsrc, src, plots);
}
