
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <errno.h>

#include "xben.h"
#include "xb_buff.h"
#include "xb_apl.h"
#include "xb_util.h"
#include "xb_menu.h"

#include "v_inp.h"
#include "u_pl_def.h"


/* Paper Dimensions in micrometers */
#define A4_X  270000
#define A4_Y  190000
#define A3_X  380000

#define DEVICETYPES 5
#define FORMATS 7

#define CM_FACTOR ((double)UNITS_PER_CM)
#define REDUCE_SCALING 0.95

static char no_yes[2][4] = {" No","Yes"};
static char devicenames[DEVICETYPES][80] = { 
                    "Unknown HPGL-Device",
                    "HP-Laserjet 4",
                    "HP-Laserjet 2P with HPGL-Cartridge",
                    "HP7550 Plotter",
                    "Postscript"};
static char formatnames[FORMATS][80] = {     
                "Din A4, Portrait, Autofeed",
                "Din A4, Landscape, Autofeed",
                "Din A4, Portrait, Manual feed",
                "Din A4, Landscape, Manual feed",
                "Din A3, Portrait, Manual feed",
                "Din A3, Landscape, Manual feed",
                "Free Format"};
static char format_allowed[DEVICETYPES][FORMATS] = 
             {{1,1,1,1,1,1,1},{1,1,0,0,0,0,0},{1,1,0,0,0,0,0},
              {1,1,1,1,1,1,0},{1,1,0,0,0,0,1}};

static void print_fromapl(beninfo *fromapl){
 printf("print_fromapl xa=%12.2f xe=%12.2f ya=%12.2f ye=%12.2f \n",
  fromapl->a_xanf,fromapl->a_xend,fromapl->a_yanf,fromapl->a_yend);

}

static void show_device(deviceinfo *config, int show_dim){
  double dimx, dimy;
  printf(" Plotdevice is ");
  switch (config->deviceclass){
     case PLOT_DEV_HPGL : {printf("%s",devicenames[config->device]);
     newline();}; break;
     case PLOT_DEV_PS   : {printf("%s",devicenames[4]);newline();}; break;
     default : {printf(" Unknown HPGL Device");newline();}; break;
  }
  printf(" Paper format is %s", formatnames[config->paper-1]);newline();
  dimx = ((double)config->paper_dimx) / ((double)UNITS_PER_CM);
  dimy = ((double)config->paper_dimy) / ((double)UNITS_PER_CM);
  if (show_dim) {
    printf(" Paper Dimensions are: %6.2f x %6.2f cm",dimx, dimy);newline();
}}

static void show_info(plotinfo *plot, deviceinfo *config){
  datatype xmin, xmax, ymin, ymax;
  int stat;

  stat = get_minmax(plot, &xmin, &xmax, &ymin, &ymax);
  printf(" Paper-Size: %5.2f x %5.2f cm",
           config->paper_dimx/CM_FACTOR, config->paper_dimy/CM_FACTOR);
  newline();
  newline();
  if (stat==XB_STAT_OK){
    printf(" Range of Plot (X) : %5.2f to %5.2f cm",
           xmin/CM_FACTOR,xmax/CM_FACTOR); newline();
    printf(" Range of Plot (Y) : %5.2f to %5.2f cm",
           ymin/CM_FACTOR,ymax/CM_FACTOR); newline();
  } else {printf("Actual buffer contains no plot instructions\n");newline();};
  printf(" Scaling (X) : %6.3f,    Shift (X) : %5.2f cm",
         plot->factor_x, plot->shift_x/CM_FACTOR);newline();
  printf(" Scaling (Y) : %6.3f,    Shift (Y) : %5.2f cm",
         plot->factor_y, plot->shift_y/CM_FACTOR);newline();
  printf(" Rotation Angle : %6.2f Degrees",plot->angle);newline();
  if ((xmin < 0) || (ymin < 0) || 
       (xmax > config->paper_dimx) || (ymax > config->paper_dimy))
    {
      newline();
      printf(" WARNING: Plot exceeds paper format !");
      newline();
    };
}

void show(plotinfo *plot, deviceinfo *config){
  newline();
  newline();
  printf("  Information about Plot");
  newline();
  printf("  ======================");
  newline();
  newline();
  printf(" Plotfile was %s",plot->filename);
  newline();
  newline();
  show_device(config,FALSE);
  show_info(plot,config);
  /*XBEN_taste_warten(TRUE);*/
}

static void ask_dimension(FILE *volterm,char what[], datatype *size){
   double tmp;
   tmp = (double)(*size) / UNITS_PER_CM;
   fprintf(volterm,"  %1s-Dimension of Paper [cm]  (D=%6.2f) : ",what,tmp);
   tmp = real_default(volterm,tmp);
   *size = (datatype)(tmp * UNITS_PER_CM);
}

void use_paper_format(deviceinfo *config){
  switch (config->paper){
    /* case PLOT_PAPER_FREE     
       : ask_dimension(volterm,"X",&config->paper_dimx); break;*/
     case PLOT_PAPER_A4P_AUTO : 
     case PLOT_PAPER_A4P      : config->paper_dimx = A4_Y; break;
     case PLOT_PAPER_A3P      : 
     case PLOT_PAPER_A4L_AUTO : 
     case PLOT_PAPER_A4L      : config->paper_dimx = A4_X; break;
     case PLOT_PAPER_A3L      : config->paper_dimx = A3_X; break;
  }
  switch (config->paper){
    /* case PLOT_PAPER_FREE     
       : ask_dimension(volterm,"Y",&config->paper_dimy); break;*/
     case PLOT_PAPER_A4L_AUTO : 
     case PLOT_PAPER_A4L      : config->paper_dimy = A4_Y; break;
     case PLOT_PAPER_A3L      : 
     case PLOT_PAPER_A4P_AUTO : 
     case PLOT_PAPER_A4P      : config->paper_dimy = A4_X; break;
     case PLOT_PAPER_A3P      : config->paper_dimy = A3_X; break;
  }
  config->xoffset = 0; config->yoffset = 0;
}

int set_format(FILE *volterm,deviceinfo *config){
  int i,ic, tmp;

  fprintf(volterm,"\n Plotdevice is Postscript (-P-)\n");
  fprintf(volterm," ===============================\n\n");
  /*show_device(config,TRUE);*/
    config->device = 4; 
    fprintf(volterm,"\tSelect Paper Format (-F-)\n\n");
    ic=config->paper;
    for (i=0; i<FORMATS; i++)
       if (format_allowed[config->device][i])
	 {
          fprintf(volterm,"     %1d - %s\n",i+1,formatnames[i]);
	 };
    if (! format_allowed[config->device][config->paper-1]) config->paper = 1;
    do{ 
      do{
        fprintf(volterm," Enter your Choice (D=%1d) : ",config->paper);    
        tmp = integer_default(volterm,config->paper);
      } while ((tmp < 1) || (tmp > FORMATS));
    } while (! format_allowed[config->device][tmp-1]);
    config->paper = tmp;
    use_paper_format(config);
    /*show_device(config,TRUE);*/
    i=0;
    if(ic!=config->paper)i=1;
    return(i);
}

static void center_plot(plotinfo *plot, deviceinfo *config,
                          datatype *move_x, datatype *move_y){
   datatype xmin, xmax, ymin, ymax, xdim, ydim;
   datatype mid_xd, mid_yd, mid_xp, mid_yp;
   get_minmax(plot, &xmin, &xmax, &ymin, &ymax);
   mid_xd = (xmin+xmax)/2;
   mid_yd = (ymin+ymax)/2;
   mid_xp = config->paper_dimx/2;  
   mid_yp = config->paper_dimy/2;
   *move_x = mid_xp-mid_xd; 
   *move_y = mid_yp-mid_yd; 
}

static void calculate_scaling(plotinfo *plot, deviceinfo *config, 
        double *factor, datatype *move_x, datatype *move_y){
   double fact_x, fact_y;      
   datatype xmin, xmax, ymin, ymax, xdim, ydim;
   get_minmax(plot, &xmin, &xmax, &ymin, &ymax);
   xdim = xmax-xmin; ydim = ymax-ymin;
   fact_x = ((double)config->paper_dimx / (double)xdim) * REDUCE_SCALING;
   fact_y = ((double)config->paper_dimy / (double)ydim) * REDUCE_SCALING;
   if (fact_x < fact_y) *factor = fact_x; else *factor = fact_y;
   *move_x = - xmin * (*factor) 
             + (double)config->paper_dimx * (1 - REDUCE_SCALING) / 2;
   *move_y = - ymin * (*factor)
             + (double)config->paper_dimy * (1 - REDUCE_SCALING) / 2;
}

void move_apl(beninfo *fromapl, datatype move_x, datatype move_y){
   fromapl->a_xanf = fromapl->a_xanf + move_x;
   fromapl->a_xend = fromapl->a_xend + move_x;
   fromapl->a_yanf = fromapl->a_yanf + move_y;
   fromapl->a_yend = fromapl->a_yend + move_y;
}
   
void scale_apl(beninfo *fromapl, double factor_x, double factor_y){
   fromapl->a_xanf = fromapl->a_xanf * factor_x;
   fromapl->a_xend = fromapl->a_xend * factor_x;
   fromapl->a_yanf = fromapl->a_yanf * factor_y;
   fromapl->a_yend = fromapl->a_yend * factor_y;
}
void reset_apl(beninfo *fromapl){
   fromapl->a_xanf = fromapl->r_xanf;
   fromapl->a_xend = fromapl->r_xend;
   fromapl->a_yanf = fromapl->r_yanf;
   fromapl->a_yend = fromapl->r_yend;
}
    
void scale(FILE *volterm,beninfo *fromapl,plotinfo *plot, deviceinfo *config){
   double move, fact_x, fact_y, angle;
   datatype move_x, move_y;
   int ok,i;
  datatype xmin, xmax, ymin, ymax;
  int stat;

   do {
     fprintf(volterm,"\tScaling of Plot\n");
     fprintf(volterm,"\t===============\n");
  stat = get_minmax(plot, &xmin, &xmax, &ymin, &ymax);
  /*show_info(plot,config);*/
     fprintf(volterm,"  1 - Fit to paper format\n");
     fprintf(volterm,"  2 - Move Plot\n");
     fprintf(volterm,"  3 - Magnify Plot\n");
     fprintf(volterm,"  4 - Rotate Plot\n");
     fprintf(volterm,"  5 - Center Plot\n");
     fprintf(volterm,"  6 - Transform Plot\n");
     fprintf(volterm,"  7 - Quit\n");
     fprintf(volterm," Enter your choice:");
     switch (integer_default(volterm,0)){
        case 0 : return;
        case 1 : /* done in two steps, since scaling affects move */
                 calculate_scaling(plot,config,&fact_x,&move_x,&move_y);
                 scale_buffer(plot,fact_x,fact_x); 
                 i=0;while(fromapl[i].iplot==i)
                           {scale_apl(&fromapl[i],fact_x,fact_x);i++;};
                 calculate_scaling(plot,config,&fact_x,&move_x,&move_y);
                 /*i=0;while(fromapl[i].iplot==i){scale_apl(&fromapl[i],fact_x,fact_x);i++;};*/
                 move_origin(plot,move_x,move_y);
                 i=0;while(fromapl[i].iplot==i)
                           {move_apl(&fromapl[i],move_x,move_y);i++;};
                 break;
        case 2 : fprintf(volterm," Shift right   [cm]  (D=0) : ");
                 move_x = real_default(volterm,0.0) * UNITS_PER_CM;
		 fprintf(volterm," Shift upwards [cm]  (D=0) : ");
                 move_y = real_default(volterm,0.0) * UNITS_PER_CM;
                 move_origin(plot,move_x,move_y); 

                 i=0;while(fromapl[i].iplot==i)
                          {move_apl(&fromapl[i],move_x,move_y);i++;};
                 break;
        case 3 : fprintf(volterm," Magnification X  (D=%5.3g) : ",1.0);
                 fact_x = real_default(volterm,1.0); if (fact_x==0.0) break;
                 fprintf(volterm,"\n Magnification Y  (D=%5.3g) : ",fact_x);
                 fact_y = real_default(volterm,fact_x); if (fact_y==0.0) break;
                 if (fabs(fact_x/fact_y-1.0) > 0.001) 
                  ok = entscheidung(volterm,
                    "Do you really want different factors for X and Y",'Y');
                 else ok = TRUE;
                 if (!ok) break;
                 scale_buffer(plot,fact_x,fact_y); 
                 i=0;while(fromapl[i].iplot==i)
                {scale_apl(&fromapl[i],fact_x,fact_y);i++;};
                 break;
        case 4 : fprintf(volterm," Rotation angle [degrees]  (D=0) : ");
                 angle = real_default(volterm,0.0); 
                 if (angle==0.0) break;
                 rotate_buffer(plot,angle); 
                 center_plot(plot,config,&move_x,&move_y); 
                 move_origin(plot,move_x,move_y);
                 i=0;while(fromapl[i].iplot==i)
                    {move_apl(&fromapl[i],move_x,move_y);i++;};
                 break; 
        case 5 : center_plot(plot,config,&move_x,&move_y); 
                 move_origin(plot,move_x,move_y);
                 i=0;while(fromapl[i].iplot==i)
                    {move_apl(&fromapl[i],move_x,move_y);i++;};
                 break;
        case 6 : transform_buffer(volterm,plot); 
                 break;
   }} while (TRUE);        
}

void scale1(beninfo *fromapl,plotinfo *plot, deviceinfo *config){
   double move, fact_x, fact_y, angle;
   datatype move_x, move_y;
   int i;
        calculate_scaling(plot,config,&fact_x,&move_x,&move_y);
        scale_buffer(plot,fact_x,fact_x); 
	/*printf("# move_x=%d move_y=%d  \n",move_x,move_y);
	 printf("# print_fromapl a_xanf=%12.2f a_xend=%12.2f  \n",
           fromapl[0].a_xanf,fromapl[0].a_xend);*/	
        i=0;while(fromapl[i].iplot==i)
		 {scale_apl(&fromapl[i],fact_x,fact_x);i++;};  /* _x.._x merkwuerdig*/
     /*printf("# print_fromapl a_xanf=%12.2f a_xend=%12.2f  \n",
       fromapl[0].a_xanf,fromapl[0].a_xend);*/	
        calculate_scaling(plot,config,&fact_x,&move_x,&move_y);
       /*i=0;while(fromapl[i].iplot==i){scale_apl(&fromapl[i],fact_x,fact_x);i++;};*/
        move_origin(plot,move_x,move_y);
        i=0;while(fromapl[i].iplot==i)
                  {move_apl(&fromapl[i],move_x,move_y);i++;};
     /*printf("# print_fromapl a_xanf=%12.2f a_xend=%12.2f  \n",
           fromapl[0].a_xanf,fromapl[0].a_xend);*/	

}

void scale5(beninfo *fromapl,plotinfo *plot, deviceinfo *config){
   double move, fact_x, fact_y, angle;
   datatype move_x, move_y;
   int i;
   datatype xmin, xmax, ymin, ymax;
   int stat;
   stat = get_minmax(plot, &xmin, &xmax, &ymin, &ymax);
   /*reset_scaling(plot);*/
   i=0;while(fromapl[i].iplot==i){reset_apl(&fromapl[i]);i++;};
   calculate_scaling(plot,config,&fact_x,&move_x,&move_y);
   /*i=0;while(fromapl[i].iplot==i){scale_apl(&fromapl[i],fact_x,fact_x);i++;};*/
    move_origin(plot,move_x,move_y);
    i=0;while(fromapl[i].iplot==i)
                  {move_apl(&fromapl[i],move_x,move_y);i++;};

}
void reset_fromxben(beninfo *fromapl,plotinfo *plot, deviceinfo *config){
   double move, fact_x, fact_y, angle;
   datatype move_x, move_y;
   int i;
   datatype xmin, xmax, ymin, ymax;
   int stat;
   reset_scaling(plot);
   i=0;while(fromapl[i].iplot==i){reset_apl(&fromapl[i]);i++;};
   /*calculate_scaling(plot,config,&fact_x,&move_x,&move_y);
   stat = get_minmax(plot, &xmin, &xmax, &ymin, &ymax);
    move_origin(plot,move_x,move_y);
    i=0;while(fromapl[i].iplot==i)
    {move_apl(&fromapl[i],move_x,move_y);i++;};*/
}
void center_fromxben(beninfo *fromapl,plotinfo *plot, deviceinfo *config){
   double move, fact_x, fact_y, angle;
   datatype move_x, move_y;
   int i;
   datatype xmin, xmax, ymin, ymax;
   int stat;
   stat = get_minmax(plot, &xmin, &xmax, &ymin, &ymax);
   center_plot(plot,config,&move_x,&move_y); 
                 move_origin(plot,move_x,move_y);
                 i=0;while(fromapl[i].iplot==i)
                    {move_apl(&fromapl[i],move_x,move_y);i++;};

		 /*printf("# print_fromapl a_xanf=%12.2f a_xend=%12.2f  \n",
		   fromapl[0].a_xanf,fromapl[0].a_xend);*/	
}

void scale_lay(beninfo *fromapl,plotinfo *plot){
   int i;
   i=0;while(fromapl[i].iplot==i){reset_apl(&fromapl[i]);i++;};
   i=0;while(fromapl[i].iplot==i)
   {scale_apl(&fromapl[i],plot[0].factor_x,plot[0].factor_y);i++;};
   i=0;while(fromapl[i].iplot==i)
   {move_apl(&fromapl[i],plot[0].shift_x,plot[0].shift_y);i++;};
}

/* File handling */



