#include <string.h>
#include "xb_write.h"
#include "xb_buff.h"
#include "xb_menu.h"
#include "xb_plot.h"
#include "xben.h"
#include "u_plot.h"
#include "v_inp.h"


#define HPGL_FACTOR 25

static void ps_bounding_box(plotinfo plot[], int bbox[]){
/* Offset of PS coordinates in cm */
#define PS_OFFSET  0.93
/* Transform internal to PS coordinates (1/72") */

#define PS_COORDS(x)\
  (x = (int)((( ((double)x) / ((double)UNITS_PER_CM) + PS_OFFSET)/2.54) *72))

  int nplot, first;
  datatype x1, x2, y1, y2, xmin, xmax, ymin, ymax;
  first = TRUE;
  for (nplot=0; nplot<MAXPLOTS; nplot++){
    if (plot[nplot].bufnum == XB_STAT_NOBUFF) continue;
    if (! plot[nplot].visible) continue;
    /*get_minmax(&plot[nplot], &x1, &x2, &y1, &y2);*/
    /*x2=x1+plot[nplot].xmax-plot[nplot].xmin;
      y2=y1+plot[nplot].ymax-plot[nplot].ymin;*/
    x1=plot[nplot].xmin;
    x2=plot[nplot].xmax;
    y1=plot[nplot].ymin;
    y2=plot[nplot].ymax;
    if (first){
      first = FALSE;
      xmin = x1; ymin = y1; xmax = x2; ymax = y2;
    }else{
      if (x1 < xmin) xmin = x1;  if (y1 < ymin) ymin = y1;
      if (x2 > xmax) xmax = x2;  if (y2 > ymax) ymax = y2;
  }}
  if (first) return;
  PS_COORDS(xmin);  PS_COORDS(xmax);  PS_COORDS(ymin);  PS_COORDS(ymax);
  bbox[0] = xmin; bbox[1] = ymin; bbox[2] = xmax; bbox[3] = ymax;
#undef PS_COORDS  
}

static void write_the_file(char filename[], plotinfo plot[], 
                           deviceinfo *config, int filetype){
   int handle, entry, bbox[4];
   char frage[80], statstr[256]; 
   static char queuename[5][20] = { "?","LASER","EPSON","HP", "LASER"};

   switch (filetype){
     case FILETYPE_HPGL :
       if (config->paper==PLOT_PAPER_FREE){
         handle = plot_open(PLOT_DEV_HPGL,
                       (int)(config->paper_dimx / HPGL_FACTOR),
                       (int)(config->paper_dimy / HPGL_FACTOR),
                       config->device, config->paper,filename,NULL); break;
       }else{
         handle = plot_open(PLOT_DEV_HPGL,0,0,
                       config->device, config->paper,filename,NULL); break;}
     case FILETYPE_PS :
       ps_bounding_box(plot, bbox); 
       if (config->paper==PLOT_PAPER_FREE){
         handle = plot_open(PLOT_DEV_PS,
                     (int)(config->paper_dimx),
                     (int)(config->paper_dimy),
                     config->device, config->paper, filename, (void *)bbox); 
       } else {
         handle = plot_open(PLOT_DEV_PS,0,0,
                     0,config->paper,filename,(void *)bbox); 
         break;
   }}
   if (handle < 0){ printf(" Error opening %s : %s",
              filename, plot_message());newline();  
   /*XBEN_taste_warten(TRUE);*/ return; }
   plot_graphmode();
   output_plot(handle,plot,config,FALSE);
   plot_close(handle);
   plot_textmode();
}

void write_plotfile(FILE *volterm,plotinfo plot[], 
                 deviceinfo *config, int filetype){
#define BELL 7
   char def_extension[80], filename[80];
   if (filetype==FILETYPE_DETECT){
       printf(" Filetype for Output (3=HPGL) (D=3) : ");
       filetype = integer_default(volterm,3);
   }
   if ((filetype != FILETYPE_PS) && (filetype != FILETYPE_HPGL)) return;
   if (filetype==FILETYPE_HPGL) strcpy(def_extension,"*.hpl");
   if (filetype==FILETYPE_PS) strcpy(def_extension,plot[0].outfile);
   if ((filetype==FILETYPE_PS) && (config->deviceclass!=PLOT_DEV_PS))
     {
       newline();
       printf("%c Warning: Selected Plotdevice is not PostScript !",BELL);
       newline();
       newline();
     };
   if ((filetype==FILETYPE_HPGL) && (config->deviceclass!=PLOT_DEV_HPGL))
     {
       newline();
       printf("%c Warning: Selected Plotdevice is not HPGL !",BELL);
       newline();
       newline();
     };

   get_filename(volterm,filename,"Plotfile for output",def_extension);
   if (strlen(filename)) write_the_file(filename,plot,config,filetype);
}


