#include "string.h"
#include "u_plot.h"
#include <X11/Xlib.h>


#include "u_pl_loc.h"

typedef struct geo {
   float akt_x, akt_y, symbol_factor;
   char name[80];
   int devtype;
   unsigned long capabilities;
   void (* ploth_line_geo)(int,int,int,int,int);
   void (* ploth_point_geo)(int,int,int);
   void (* ploth_fillrectangle_geo)(int,int,int,int,int);
   void (* ploth_fillpolygon_geo)(int,int,XPoint*);
   void (* ploth_forgrcolor_geo)(int,int,XColor*,unsigned long);
   void (* ploth_drawstring_geo)(int,int,int,char*);
} geotype;
typedef geotype *geoptr;

#include <stdio.h>
#include <stdlib.h>
#include "v_plotstr.h"

#define A4_X  19.
#define A4_Y  27.

static float FACTOR=40.;     /* ~625/19   */
static float shiftx=0.,shifty=0.;
static FILE *pfilenumb;
static int file_is_open= FALSE;
static int rotate, devsizex, devsizey, mirrory,bbox[4], nplot;

static void write_point(char *cmd, int x, int y){
     int tmp;
     
     if(mirrory) y = devsizey - y;
     if (rotate) { tmp = x; x = devsizey-y; y = tmp; }
   fprintf(pfilenumb," %g %g %s\n",x/FACTOR+shiftx,y/FACTOR+shifty,cmd);
}

void plotd_line_ps(int handle, int X0, int Y0, int X1, int Y1){
   write_point("MT",X0,Y0);
   write_point("DT\nST",X1,Y1);
}
void plotd_point_ps(int handle, int X, int Y){
   plotd_line_ps(handle,X,Y,X,Y);
}

void plotd_polyline_ps(int handle, int pts, int X[], int Y[]){
  int i;     
  write_point("MT",X[0],Y[0]);
  for (i=1; i<pts; i++) write_point("DT",X[i],Y[i]);
  fprintf(pfilenumb,"ST\n");
}
void plotd_polylinevc_ps(int handle, int pts, XPoint *itmp){
  int i;    
  write_point("MT",itmp[0].x,itmp[0].y);
  for (i=1; i<pts; i++) write_point("DT",itmp[i].x,itmp[i].y);
  fprintf(pfilenumb,"ST\n");
}
int plotd_setfgbyname_ps(int handle, char *color){
/* set foreground color */
    unsigned int fg;
    /*printf("color=%s \n", color);*/
    fg=0;
    while(strcmp(color,win.tabcolor[fg]) !=0 && fg < 64)fg++;
    /*printf("fg=%d  win.tabcolor=%s \n", fg,win.tabcolor[fg]);*/
    fprintf(pfilenumb,"SP%d\n",fg);
    return fg;
}
int plotd_setbgbyname_ps(int handle, char *color){
/* set foreground color */
    unsigned int bg;
    bg=0;
    while(strcmp(color,win.tabcolor[bg]) !=0 && bg < 64)bg++;
    fprintf(pfilenumb,"SP%d\n",bg);
    return bg;
}
void plotd_forgrcolor_ps(int handle,int fg,XColor colaray[],unsigned long j){
    fprintf(pfilenumb,"SP%d\n",fg);
}
void plotd_polysymbol_ps(int handle, int pts, symboltyp sym,int x[], int y[]){
   int a1,a2,b1,b2,i,k;
   for (i=0; i<pts; i++)
     for(k=0; k<sym.vec; k++){       
       a1= x[i]+sym.x1[k];			     
       a2= x[i]+sym.x2[k];			     
       b1= y[i]+sym.y1[k];	     
       b2= y[i]+sym.y2[k];
       plotd_line_ps(handle, a1, b1, a2, b2);
     }
}

void plotd_drawstring_ps(int handle, int x, int y, char txt[]){
   int tmp;
    if(mirrory) y = devsizey - y;
    fprintf(pfilenumb,"  %g  %g  %g DI\n",0.15,0.2,0.0);
   
   if (rotate){
     tmp = x; x =devsizey- y; y = tmp;
     fprintf(pfilenumb," %g %g %g (%s) LB\n",x/FACTOR+shiftx,y/FACTOR+shifty,90.0,txt);
   }else
     fprintf(pfilenumb," %g %g %g (%s) LB\n",x/FACTOR+shiftx,y/FACTOR+shifty,0.0,txt);
}

void plotd_writechar_ps(int handle, int x, int y, char *txt){
   int tmp;
   fprintf(pfilenumb,"  %g  %g  %g DI\n",0.2,0.3,0.0);
    
   if (rotate){
     tmp = x; x =devsizey- y; y = tmp;
     fprintf(pfilenumb," %g %g %g (%s) LB\n",x/FACTOR+shiftx,y/FACTOR+shifty,90.0,txt);
   }else
     fprintf(pfilenumb," %g %g %g (%s) LB\n",x/FACTOR+shiftx,y/FACTOR+shifty,0.0,txt);
}
void plotd_fillpolygon_ps(int handle,int pts,XPoint *itmp){
  int i;    
  write_point("MT",itmp[0].x,itmp[0].y);
  for (i=1; i<pts; i++) write_point("DT",itmp[i].x,itmp[i].y);  
  fprintf(pfilenumb,"fill\n");
  fprintf(pfilenumb,"ST\n");
}

void plotd_fillrectangle_ps(int handle, int X0, int Y0, int w, int h){   
  write_point("MT",X0,Y0);
  write_point("DT",X0+w,Y0);
  write_point("DT",X0+w,Y0+h);
  write_point("DT",X0,Y0+h);
  write_point("DT",X0,Y0);
  fprintf(pfilenumb,"fill\n");
  fprintf(pfilenumb,"ST\n");
}

static void filed_close_ps(int handle){
   fprintf(pfilenumb,"grestore showpage\n");
   fclose(pfilenumb);
   file_is_open = FALSE;
}

void plotd_func_ps(handletyp *h,int vector,int trm,int numpl,int numplot){
  FILE *headerfile;
   char headerfilename[128];
   char tmp[81];
   float hh;
   int i;

   devsizex= win.sizex[win.ksphandle-1];
   devsizey= win.sizey[win.ksphandle-1];
   if(numpl == 1){

   bbox[0]=20; bbox[1]=20; 
   rotate=0;
   if((trm == 108 || trm == 76) && devsizex*0.9> devsizey) rotate=1;
   if(rotate){FACTOR=devsizex/A4_Y;
              bbox[2]=620*devsizey/devsizex*A4_Y/A4_X;
              bbox[3]=840;
   }else{     FACTOR=devsizex/A4_X;
              bbox[2]=590;
              bbox[3]=620*devsizey/devsizex;}

   if(numplot > 1){
              bbox[2]=590;
              bbox[3]=840;
              shifty=0;
   }



   strcpy(headerfilename,"PLOT_HEADER.PS");
   headerfile = fopen(headerfilename,"r"); 
   /*i=strlen(plot.datfile)-2;
     printf("strlen(plot.datfile)-2=%d plot.datfile=%s \n",i,plot.datfile);*/
   printf("plot.datfile=%s \n",plot.datfile);
   pfilenumb = fopen(plot.datfile,"w");
   if((pfilenumb)==NULL)printf("%%!PS\n");
   fprintf(pfilenumb,"%%!PS\n");
   fprintf(pfilenumb, "%%%%BoundingBox: %d %d %d %d\n", 
             bbox[0],bbox[1],bbox[2],bbox[3]);
   while (fgets(tmp,80,headerfile)) fputs(tmp,pfilenumb);
   fclose(headerfile);
   if(numplot > 0){nplot=numplot+1;
                   printf(" %d sets to be plotted to A4 page\n",numplot);
                   printf("set number %d  : select number 2\n",1);}
   }
   h->ploth_line         =   plotd_line_ps;
   h->ploth_polyline     =   plotd_polyline_ps;
   h->ploth_point        =   plotd_point_ps;
   h->ploth_linevc       =   plotd_line_ps;
   h->ploth_polylinevc   =   plotd_polylinevc_ps;
   h->ploth_pointvc      =   plotd_point_ps;
   h->ploth_polysymbol   =   plotd_polysymbol_ps;
   h->ploth_drawstring   =   plotd_drawstring_ps;
   h->ploth_writechar    =   plotd_writechar_ps;
   h->ploth_setfgbyname  =   plotd_setfgbyname_ps;
   h->ploth_setbgbyname  =   plotd_setbgbyname_ps;
   h->ploth_fillpolygon  =   plotd_fillpolygon_ps;
   h->fileh_close         =  filed_close_ps;   
   

   mirrory=vector;
   //printf("sizex=%d \n", devsizex); printf("sizey=%d \n", devsizey);
   if(numplot > 0){hh=A4_Y/(nplot-1);
              shifty=(numplot-1)*hh;
              hh=devsizey/hh;
              FACTOR=devsizex/A4_X;
              /*printf("hh=%g  FACTOR=%g \n",hh, FACTOR);*/
              if(FACTOR< hh) FACTOR=hh;
              shiftx=(A4_X-devsizex/FACTOR)/2.;
              hh=A4_Y/(nplot-1);
              if(numplot > 1)shifty=shifty+(hh-devsizey/FACTOR)*0.75+1.;
   }
   if(numpl < 0){if(numplot>1)
                 printf("set number %d : select number %d \n",
                                       nplot-numplot,nplot-numplot+1);else
           printf("  last set %d (recover by left mouse click)\n",nplot-numplot);}
   }


void plotd_func_psgeo(geotype *geo, XColor colaray[16],int colnum, 
                       int vector,int trm, int numpl,int numplot,
                                  int winh, int winw){
   FILE *headerfile;
   char headerfilename[128];
   char tmp[120];
   float hh;
   int i;

   /*for(i=0;i<colnum+1;i++)
       printf("red =%d  green =%d blue =%d \n",
       colaray[i].red,colaray[i].green,colaray[i].blue);*/

    devsizex= winw;
    devsizey= winh;
   if(numpl == 1){

   bbox[0]=20; bbox[1]=20; 
   rotate=0;
   if((trm == 108 || trm == 76) && devsizex*0.9> devsizey) rotate=1;
   if(rotate){FACTOR=devsizex/A4_Y;
              bbox[2]=620*devsizey/devsizex*A4_Y/A4_X;
              bbox[3]=840;
   }else{     FACTOR=devsizex/A4_X;
              bbox[2]=590;
              bbox[3]=620*devsizey/devsizex;}

   if(numplot > 1){
              bbox[2]=590;
              bbox[3]=840;
              shifty=0;
   }

   strcpy(headerfilename,"PLOT_HEADER.PS");
   headerfile = fopen(headerfilename,"r"); 

   pfilenumb = fopen("geo.ps","w");
   fprintf(pfilenumb,"%%!PS\n");
   fprintf(pfilenumb, "%%%%BoundingBox: %d %d %d %d\n", 
             bbox[0],bbox[1],bbox[2],bbox[3]);
   while (fgets(tmp,80,headerfile)){
   if(tmp[2] == 'P' && tmp[3] == '0') goto endodheader;
              else fputs(tmp,pfilenumb);}
   endodheader:
/*/SP0  { 0.03 cm setlinewidth 0.000 0.000 0.000 setrgbcolor } def*/
   for(i=0;i<colnum+1;i++){
   sprintf(tmp,"/SP%d { 0.03 cm setlinewidth %6.3f %6.3f %6.3f setrgbcolor } def\n",
	  i,(float)colaray[i].red/65535,(float)colaray[i].green/65535,
            (float)colaray[i].blue/65535);
   
   fputs(tmp,pfilenumb);}
   fclose(headerfile);

   if(numplot > 0){nplot=numplot+1;
                   printf(" %d sets to be plotted to A4 page\n",numplot);
                   printf("set number %d \n",1);}
   }
   geo->ploth_line_geo          =   plotd_line_ps;
   geo->ploth_point_geo         =   plotd_point_ps;
   geo->ploth_fillrectangle_geo =   plotd_fillrectangle_ps;
   geo->ploth_fillpolygon_geo   =   plotd_fillpolygon_ps;
   geo->ploth_forgrcolor_geo    =   plotd_forgrcolor_ps;   
   geo->ploth_drawstring_geo    =   plotd_drawstring_ps;

   mirrory=vector;
   /*printf("sizex=%d \n", devsizex);
     printf("sizey=%d \n", devsizey);*/
   if(numplot > 0){hh=A4_Y/(nplot-1);
              shifty=(numplot-1)*hh;
              hh=devsizey/hh;
              FACTOR=devsizex/A4_X;
              /*printf("hh=%g  FACTOR=%g \n",hh, FACTOR);*/
              if(FACTOR< hh) FACTOR=hh;
              shiftx=(A4_X-devsizex/FACTOR)/2.;
              hh=A4_Y/(nplot-1);
              if(numplot > 1)shifty=shifty+(hh-devsizey/FACTOR)*0.75+1.;
   }
   if(numpl < 0){if(numplot>1)printf("set number %d \n",nplot-numplot);
   else printf("  last set %d \n",nplot-numplot);}

   if(numplot == 0)numplot=-1;
   }

