
subroutine LORGAU(gamnat,e_xsi,srw,w_Lorentz,w_Gauss,&
                                              iflag_Voigt,ipos,nLG)
      use glbconst, only: maxscrd
      use comthe, only: mLG,mGa,array_fr
      implicit none 
!     version August 2013
!     calculation of the Lorentz-Gauss functions                        
      real,    intent(in) :: gamnat,e_xsi,srw,w_Lorentz,w_Gauss 
      integer, intent(in) :: nLG,iflag_Voigt,ipos
                                                                        
      complex  aLOR
      real     Gauss,dumscrd
      integer, parameter:: idumscrd=maxscrd-(mLG+mGa+1)*2-(mGa+2) 
      common/scrd/aLOR(0:mLG+mGa),Gauss(0:mGa+1),dumscrd(idumscrd) 
!                 aLOR and Gauss is only used in case of iflag_Voigt=1: 
!                 calculation of the Voigt profile by expicit convolution
      integer  ipLG,i,ibg,j,ij,n
      logical  b_Lorentz,b_Gauss,b_gamnat,b_xsi,b_srw,b_nLG,           &
               b_Gaussgt0,b_flag_Voigt,b_com,b_ipos        

      real     w_aLorentz,w_aGauss,wpi,dxc,xc,yc,vxy,dvxy,              &
               halpda1,halpda2,hbeta1,hbeta2,beta1,beta2,beta3,beta4,   &
               alpha1,alpha2,alpha3,alpha4,widthg,widthl2,width2,       &
               disp,hp,h0,h1                                            
      real, parameter:: a1=-1.2150,a2=-1.3509,b1= 1.2359,b2= 0.3786,    &
                        c1=-0.3085,c2= 0.5906,d1= 0.0210,d2=-1.1858,    &
                        d12=0.0105,d22=-0.5928,wln2=0.832554611
      complex  hc,hc1,hxsi 
                                             
!     write(6,'(2a,/,2l5,4f10.3,5i6)') 'LORGAU b_Lorentz,b_Gauss,', &  
!     'gamnat,w_Lorentz,w_Gauss,srw,iflag_Voigt,nLG,ipLG',          & 
!     b_Lorentz,b_Gauss,gamnat,w_Lorentz,w_Gauss,srw,               & 
!     iflag_Voigt,nLG,ipLG                          
                                                                         
      ipLG=ipos-6
      h0=gamnat/100. 
      h1=h0/100. 
      b_gamnat =abs(gamnat-real(array_fr(ipLG+1))).lt.h0 
      b_xsi    =abs(e_xsi-aimag(array_fr(ipLG+1))).lt.h0 
      b_Lorentz=abs(w_Lorentz-real(array_fr(ipLG+2))).lt.h0 
      b_Gauss  =abs(w_Gauss-aimag(array_fr(ipLG+2))).lt.h0 
      b_srw   =abs(srw-real(array_fr(ipLG+3))).lt.h1 
      b_nLG    =nLG  .eq.int(aimag(array_fr(ipLG+3))+0.01) 
      b_com=b_gamnat.and.b_xsi.and.b_Lorentz.and.b_Gauss.and.b_srw.and.b_nLG
!     write(6,'(a,6f15.6)')' w_Lorentz,w_Gauss=',w_Lorentz,w_Gauss 
!     write(6,'(6f10.1)')(array_fr(ipLG+j),j=0,6) 
                                                                        
      b_flag_Voigt=iflag_Voigt.eq.int(aimag(array_fr(ipLG+4))+0.01) 
      b_ipos=ipos.eq.int(real(array_fr(ipLG+4))+0.01) 

      w_aLorentz=abs(w_Lorentz)+1. 
      w_aGauss=abs(w_Gauss) 
      b_Gaussgt0=w_aGauss.gt.0.01 
!     write(6,'(a,12L4)')'b_com,b_gamnat...=',             & 
!          b_gamnat,b_xsi,b_Lorentz,b_srw,b_nLG,b_flag_Voigt,&
!          b_ipos,b_Gaussgt0                     

!      if iflag_Voigt is not defined iflag_Voigt is =0 by default, so that the 
!      approximation is used and not the time consuming integration 
                                            
      if(iflag_Voigt.eq.0)then 
       if(b_Gaussgt0)then 
        if(.not.(b_com.and.b_flag_Voigt.and.b_ipos)) then 
!        Voigt profile according to A.B. McLean, C.E.J. Mitchell and    
!        D.M. Swantston, J.Electron Spectroscopy 69(1994)125-132        
         wpi=1.772453851/w_aLorentz 
         hxsi=cmplx(1.,2*e_xsi)*wpi 
         dxc=2.0*wln2*srw/gamnat/w_aGauss 
         yc=wln2*w_aLorentz/w_aGauss 
         halpda1=c1*(yc-a1)-d1*b1 
         halpda2=c2*(yc-a2)-d2*b2 
         hbeta1=(yc-a1)**2 
         hbeta2=(yc-a2)**2 
         xc=-dxc 
         do  n=0,nLG 
          xc=xc+dxc 
          beta1=hbeta1+(xc-b1)**2 
          beta3=hbeta1+(xc+b1)**2 
          beta2=hbeta2+(xc-b2)**2 
          beta4=hbeta2+(xc+b2)**2 
          alpha1=(halpda1+d1*xc)/beta1 
          alpha3=(halpda1-d1*xc)/beta3 
          alpha2=(halpda2+d2*xc)/beta2 
          alpha4=(halpda2-d2*xc)/beta4 
          vxy=alpha1+alpha2+alpha3+alpha4 
          dvxy=( d12-alpha1*(xc-b1))/beta1+                             &
               (-d12-alpha3*(xc+b1))/beta3+                             &
               ( d22-alpha2*(xc-b2))/beta2+                             &
               (-d22-alpha4*(xc+b2))/beta4                              
           array_fr(ipos+n)=cmplx(xc*vxy+dvxy,-yc*vxy)*hxsi 
         end do 
                                                                        
!        write(6,'(a)')'## iflag_Voigt.eq.0 &  b_Gaussgt0'               
         array_fr(ipLG+1)=cmplx(gamnat,e_xsi) 
         array_fr(ipLG+2)=cmplx(w_Lorentz,w_Gauss) 
         array_fr(ipLG+3)=cmplx(srw,float(nLG)) 
         array_fr(ipLG+4)=cmplx(float(ipos),float(iflag_Voigt)) 
        end if 
       else 
!       Lorentz                                                         
        if(.not.(b_com.and.b_ipos)) then 
         widthL2=w_aLorentz*gamnat/2. 
         disp=srw/widthL2 
         hp=-disp 
         hxsi=cmplx(1.,2*e_xsi)/w_aLorentz 
         do  n=0,nLG 
          hp=hp+disp 
          array_fr(ipos+n)=cmplx(hp,-1.)/(1.0+hp*hp)*hxsi 
         end do 

!        write(6,'(a,i8,2L5)')' iflag_Voigt.eq.0 &  w_Gauss=0',ipLG,b_com
         array_fr(ipLG+1)=cmplx(gamnat,e_xsi) 
         array_fr(ipLG+2)=cmplx(w_Lorentz,w_Gauss) 
         array_fr(ipLG+3)=cmplx(srw,float(nLG)) 
         array_fr(ipLG+4)=cmplx(float(ipos),float(iflag_Voigt)) 
        end if 
       end if 
      else if(iflag_Voigt.eq.1)then 
!      Lorentz curve is needed from 0 to nLG+mGa                        
        widthL2=w_aLorentz*gamnat/2. 
        disp=srw/widthL2 
        hp=-disp 
        hxsi=cmplx(1.,2*e_xsi)/w_aLorentz 
        do  n=0,nLG+mGa 
         hp=hp+disp 
         aLOR(n)=cmplx(hp,-1.)/(1.0+hp*hp)*hxsi 
        end do 

        widthG=w_aGauss*gamnat 
        ibg=(6.*widthG/srw+1.)/2.    ! 0.0005 precission 
        ibg=2*ibg 
                                                                        
        if(ibg.gt.0) then 
         if(ibg.gt.mGa) ibg=mGa 
!        -ibg--------0--------ibg           Gauss                       
                                                                        
         width2=widthG/wln2/2.0  ! units of gamma natural  
         disp=srw/width2 
         h0=-disp 
         do i=0,ibg+1 
          h0=h0+disp 
          Gauss(i)=exp(-h0**2) 
         end do 
          h1=0. 
         do j=1,ibg-1,2 
          h1=h1+ Gauss(j)*2. 
         end do 
         do j=2,ibg-2,2 
          h1=h1+ Gauss(j) 
         end do 
         h1=(h1*2.+Gauss(0)+Gauss(ibg))*2. 
         h1=2.*h1/3. 
         do i=0,ibg+1 
          Gauss(i)=Gauss(i)/h1 
         end do 
                                                                        
!        convolution with exp(-x**2/(sGgL*gamnat)**2)                   
!                                                                       
!        0------------------------------------nLG---nLor                
!                                       ibg----0----ibg                 
!        0------------------------------------nLG                       
!        this convolution is calculated with stepwidth srw             
                                                                        
         do i=0,nLG 
          hc=(0.,0.) 
          do j=-ibg+1,-1,2 
           ij=i+j 
           if(ij.gt.0) then 
            hc=hc+Gauss(-j)*aLOR(ij)*2. 
           else 
            hc=hc-Gauss(-j)*conjg(aLOR(ij))*2. 
           end if 
          end do 
          do j=1,ibg-1,2 
           hc=hc+Gauss(j)*aLOR(i+j)*2. 
          end do 
          do j=-ibg+2,-2,2 
           ij=i+j 
           if(ij.gt.0) then 
            hc=hc+Gauss(-j)*aLOR(ij) 
           else 
            hc=hc-Gauss(-j)*conjg(aLOR(ij)) 
           end if 
          end do 
          do j=0,ibg-2,2 
           hc=hc+Gauss(j)*aLOR(i+j) 
          end do 
          ij=i-ibg 
          if(ij.gt.0) then 
           hc1=aLOR(ij) 
          else 
           hc1=-conjg(aLOR(ij)) 
          end if 
                                                                        
          hc=hc*2.+Gauss(ibg)*(hc1+aLOR(i+ibg)) 
!         cLG(i,iset)=cmplx(hr,hi)/3.                                   
          array_fr(ipos+i)=hc/1.5 
         end do 
        end if  ! if ibg >0  

!       write(6,'(a)')'end iflag_Voigt.eq.0 &  w_Gauss >0'               
        array_fr(ipLG+1)=cmplx(gamnat,e_xsi) 
        array_fr(ipLG+2)=cmplx(w_Lorentz,w_Gauss) 
        array_fr(ipLG+3)=cmplx(srw,float(nLG)) 
        array_fr(ipLG+4)=cmplx(float(ipos),float(iflag_Voigt)) 
      end if ! iflag_Voigt=1 
                                                                        
!     If a Gauss broadening is used (simulating                         
!     a hyperfine field distribution), the sum of the Gauss function    
!     values (weighting factors)  has to be 1. 
!     Norm of the Lorentz-curve: Int -oo to +oo is gamma*pi/2
!        hc=(0.,0.) 
!         do n=-nLG+1,nLG-1,2 
!          hc=hc+ array_fr(ipos+abs(n))*4 
!         end do 
!         do n=-nLG+2,nLG-2,2 
!          hc=hc+array_fr(ipos+abs(n))*2 
!        end do
!          hc=hc+ 2*array_fr(ipos+nLG); hc=hc/3. 
!Print*,'LG lorentz sum=',2*hc*srw/gamnat/3.14159                                                                        

      return 
      END subroutine LORGAU

      
     subroutine sum2datasets(namedata,datfilesp,info)
      use glbconst, only: maxthss,maxfr1,maxfr2,ivstrf,nwr,nrd,infotext
      use comdat,   only: ks_set,mfpt,ichanks,b_theory
      use comthe,   only: real_fr,thfk
      implicit none 

      character(len=132),intent(in)   :: namedata,datfilesp
      integer,           intent(out)  :: info
      character(len=132),dimension(7):: chin,chin2,helpar,queryar
      character(len=132):: chcom,ch132,filename,querytype
      character(len=32) :: filetype,xaxis,yaxis,format
      character(len=16) :: exten
      character(len=1)  :: ch1
 
      logical b_file,b1_y,b1_xy,b1_xydy,b2_y,b2_xy,b2_xydy,b_plus
      integer il,nchannel,j,k,kk,iout,lq,lk,lqs,npl,irw
      real    sum,sumdiff,sumth
      real, dimension(:), allocatable:: x1,set1,dy,set2,out
      character(len=80):: chdf

      write(nwr,'(a,i2,a)')' data set number ',ks_set,' is loaded'
  1     npl=80
        lq=6
        lk=lq 
        chdf='e' 
        chdf(80:80)='#' 
        chin='data set manipulation' 
        helpar(1)='effi/counts/summation' 
        helpar(2)='effi/summation_2sp' 
        helpar(3)='effi/subtraction_2sp' 
        helpar(4)='effi/contraction_sp'
        helpar(5)='effi/odd_even_selection'
        helpar(6)='effi/foldingpoint' 
        queryar(1)='summation of counts of data set n defined by {pl sn}' 
        queryar(2)='summation of 2 data sets read from file' 
        queryar(3)='subtraction of 2 data sets read from file' 
        queryar(4)='contraction to half the number of channels {pl sn}'
        queryar(5)='selection odd/even channels'
        queryar(6)='calculation of standard folding point'        
        call whichcase(lq,lk,chin,chdf,helpar,queryar,7)
        if(lq.eq.0) then 
         info=1 
         infotext(1)='effi/query' 
         infotext(2)='no substring selected or exited with (q/*)' 
         infotext(3)=' ' 
          goto 1 
        end if 

      if(lq.eq.1)then
       print '(a,i2)',' data set number ',ks_set
       print '(a)',' summation of counts in the data region(s)'
       sum=0. ; lqs=0
       do j=2,ichanks(0),2 
        print '(a,i4,a,i4)',' from ',ichanks(j-1),' to ',ichanks(j)
        lqs=lqs+ichanks(j)-ichanks(j-1) 
        do k=ichanks(j-1),ichanks(j) 
         sum=sum+real_fr(mfpt+k)
        end do 

        if(b_theory)then
         sumdiff=0.; sumth=0.0
         do k=ichanks(j-1),ichanks(j) 
          sumdiff=sumdiff-real_fr(mfpt+k)+thfk(k)
          sumth=sumth+thfk(k)           
         end do 
        end if
       end do
       print '(/,a,i12)',' number of channels in the data region=', lqs        
       print '(a,f15.0)',' sum inside the region {dr}           =',sum
       print '(a,i12)',  ' average number of counts             =',int(sum/lqs)
       if(b_theory)then 
        print '(a,f15.1)',' tint(t_eff=0)=Sum_i base*geo_i       =',sumth
        print '(a,f15.1)',' Sum_i (exp-theo)_i                   =',sumdiff/lqs
        print '(a,f15.4)',' Sum_i (exp-theo)_i/tint              =',sumdiff/sumth
       end if
       return
      end if
       irw=15

      if(lq.eq.2.or.lq.eq.3) then
       b_plus=.true.
       if(lq.eq.3) b_plus=.false.
          
       allocate(out(32))
       info=0
       chin=' '
       filename='set1.dat' 
       chcom='effi/summation' 
       filetype='first set: counts/channel' 
       exten='dat' 
   10  call rdfname(filetype,filename,exten,chcom,npl,nwr)
       if(exten.eq.'*')then
        deallocate(out)
        return
       end if
       inquire(file=filename,exist=b_file) 
       if(b_file) then 
        open(unit=irw,file=filename                                    &
                 ,status='old'                                        &
                 ,access='sequential'                                 &
                 ,form='formatted')                                    
       else
        print'(a)','file '//TRIM(filename)//' not found' 
        goto 10 
       end if 
      
       read(irw,'(a)')chin(1) ; print'(a)',TRIM(chin(1))
       read(irw,'(a)')chin(2) ; print'(a)',TRIM(chin(2))
       j=index(chin(2),')')       
       format=chin(2)(1:j)
       
       il=1
       do while(il.le.4)
        read(irw,'(a)')chin(il+2) ; print'(a)',TRIM(chin(il+2))        
        j=index(chin(il+2),'nu_channel')
        if(j.gt.0) then
         ch132=chin(il+2)
         call trlch(npl,ch132,iout,out)
         nchannel=out(1)
        else
         if(index(chin(il+2),'=').eq.0)then
          backspace(irw); chin(il+2)=' '; il=5
         end if
        end if
        il=il+1
       end do
       b1_y=.false.
       if(index(chin(2),' y').gt.0)b1_y=.true.
       b1_xy=.false.
       if(index(chin(2),' x,y').gt.0)then
          b1_xy=.true.;  b1_y=.false.
       end if   
       b1_xydy=.false.
       if(index(chin(2),' x,y,dy').gt.0)then
          b1_xydy=.true.;  b1_xy=.false.
       end if
       allocate(x1(nchannel),set1(nchannel), &
                dy(nchannel),set2(nchannel))

       if(b1_y) read(irw,format)(set1(k),k=1,nchannel)
       if(b1_xy) read(irw,format)(x1(k),set1(k),k=1,nchannel)
       if(b1_xydy)read(irw,format)(x1(k),set1(k),dy(k),k=1,nchannel)
       
       read(irw,'(a)')yaxis
       read(irw,'(a)')xaxis
       close(unit=irw)
   
       chcom='effi/summation' 
       filetype='second set: counts/channel' 
       exten='dat'
       filename='set2.dat'  
   20  call rdfname(filetype,filename,exten,chcom,npl,nwr)
      
        inquire(file=filename,exist=b_file) 
        if(b_file) then 
         open(unit=irw,file=filename                                    &
                  ,status='old'                                        &
                  ,access='sequential'                                 &
                  ,form='formatted')                                    
        else
         print'(a)','file '//TRIM(filename)//' not found' 
         goto 20 
        end if 
      
        chin2=' '
        il=1
        do while(il.le.6)
         read(irw,'(a)')chin2(il) ; print'(a)',TRIM(chin2(il))
         if(index(chin2(il),'=').eq.0.and.il.gt.3)il=7
         il=il+1
        end do
        backspace(irw)
        j=index(chin2(2),')')       
        format=chin2(2)(1:j)

       b2_y=.false.
       if(index(chin2(2),' y').gt.0)b2_y=.true.
       b2_xy=.false.
       if(index(chin2(2),' x,y').gt.0)then
          b2_xy=.true.;  b2_y=.false.
       end if   
       b2_xydy=.false.
       if(index(chin2(2),' x,y,dy').gt.0)then
          b2_xydy=.true.;  b2_xy=.false.
       end if
      
       if(b2_y) read(irw,format)(set2(k),k=1,nchannel)
       if(b2_xy) read(irw,format)(x1(k),set2(k),k=1,nchannel)
       if(b2_xydy) read(irw,format)(x1(k),set2(k),dy(k),k=1,nchannel)
       
       if(b_plus) then
         do k=1,nchannel
          set1(k)=set1(k)+set2(k) 
         end do
       else
         do k=1,nchannel
          set1(k)=set1(k)-set2(k) 
         end do
       end if
      
         close(unit=irw)
      
         chcom='effi/summation'
       if(b_plus)then  
        filetype='sum of sets: counts/channel' 
        filename='sum.dat'
        else
         filetype='set1-set2: counts/channel' 
         filename='diff.dat'      
        end if
        exten='dat' 
       call rdfname(filetype,filename,exten,chcom,npl,nwr)
      
       open(unit=irw,file=filename                                    &
                ,status='unknown'                                    &
                ,access='sequential'                                 &
                ,form='formatted')                                    
      
       querytype='which header for the sum/diff: [of the first set]'
       ch1='y' 
       chcom='effi/sum/set1_2' 
       call yesno(querytype,ch1,chcom,npl,nrd,nwr) 
       if(ch1.eq.'n')then
        chin=' '
        j=1
        do while(chin2(j).ne.' ')
         chin(j)=chin2(j)
         j=j+1
        end do  
       end if 

       querytype=' the format of the count number may not be sufficient'
       ch132=chin(2)
       chcom='effi/sum/set1_2/format' 
       call askstring(querytype,ch132,chcom,npl,nwr)       
       chin(2)=ch132 
       j=index(ch132,')')
       format=ch132(1:j)
       j=1
       do while(chin(j).ne.' ')
        write(irw,'(a)')TRIM(chin(j))         
        j=j+1
       end do  
       b1_y=.false.
       if(index(chin(2),' y').gt.0)b1_y=.true.
       b1_xy=.false.
       if(index(chin(2),' x,y').gt.0)then
          b1_xy=.true.;  b1_y=.false.
       end if   
       b1_xydy=.false.
       if(index(chin(2),' x,y,dy').gt.0)then
          b1_xydy=.true.;  b1_xy=.false.
       end if
       if(b1_y) write(irw,format)(set1(k),k=1,nchannel)
       if(b1_xy) write(irw,format)(x1(k),set1(k),k=1,nchannel)
       if(b1_xydy) write(irw,format)(x1(k),set1(k),dy(k),k=1,nchannel)

       write(irw,'(a)')yaxis
       write(irw,'(a)')xaxis
       write(irw,'(a)')' '
       write(irw,'(a)')'##'
       close(unit=irw)                
      
       deallocate(x1,set1,dy,set2,out)
       return
      end if
    

      if(lq.eq.4)then
       allocate(set1(2048))  
       chcom='effi/contraction_sp' 
       filetype='counts/channel' 
       exten='dat'
       j=index(datfilesp,'.')-1
       filename=datfilesp(:j)//'_r2.dat'
       call rdfname(filetype,filename,exten,chcom,npl,nwr)
      
       open(unit=irw,file=filename                 &
                ,status='unknown'                  &
                ,access='sequential'               &
                ,form='formatted')                  
       kk=0
       do j=2,ichanks(0),2 
          print '(a,i4,a,i4)',' from ',ichanks(j-1),' to ',ichanks(j)
       do k=ichanks(j-1),ichanks(j),2
         kk=kk+1
         set1(kk)= real_fr(mfpt+k)+real_fr(mfpt+k+1)
       end do
       end do

       querytype=' the format of the counts'
       ch132='(5x,f10.0)'
       chcom='effi/counts/format' 
       call askstring(querytype,ch132,chcom,npl,nwr)       
       format=ch132 
       write(irw,'(a)')'1/2:'//TRIM(namedata)
       write(irw,'(a)')TRIM(format)//'    y'       
       write(irw,'(a,i5)')'nu_channel=',kk
       write(irw,format)(set1(k),k=1,kk)
       write(irw,'(a)')'counts'
       write(irw,'(a)')'channel'
       write(irw,'(a)')' '
       write(irw,'(a)')'##'
       close(unit=irw)
       deallocate(set1)      

       return      
      end if

      if(lq.eq.5)then
       allocate(set1(2048))  
       chcom='effi/selection_sp' 
       filetype='odd counts/channel' 
       exten='dat'
       j=index(datfilesp,'.')-1
       filename=datfilesp(:j)//'_1.dat'
       call rdfname(filetype,filename,exten,chcom,npl,nwr)
      
       open(unit=irw,file=filename                 &
                ,status='unknown'                  &
                ,access='sequential'               &
                ,form='formatted')                  

       filetype='even counts/channel' 
       exten='dat'
       j=index(datfilesp,'.')-1
       filename=datfilesp(:j)//'_2.dat'
       call rdfname(filetype,filename,exten,chcom,npl,nwr)
      
       open(unit=16,file=filename                 &
                ,status='unknown'                  &
                ,access='sequential'               &
                ,form='formatted')                  
       kk=0
       do j=2,ichanks(0),2 
        print '(2(a,i4),a)',' from ',ichanks(j-1),' to ',ichanks(j)-1,' step 2'
       do k=ichanks(j-1),ichanks(j),2
         kk=kk+1
         set1(kk)= real_fr(mfpt+k)
       end do
       end do

       querytype=' the format of the counts'
       ch132='(5x,f10.0)'
       chcom='effi/counts/format' 
       call askstring(querytype,ch132,chcom,npl,nwr)       
       format=ch132 
       write(irw,'(a)')'odd:'//TRIM(namedata)
       write(irw,'(a)')TRIM(format)//'    y'       
       write(irw,'(a,i5)')'nu_channel=',kk
       write(irw,format)(set1(k),k=1,kk)
       write(irw,'(a)')'counts'
       write(irw,'(a)')'channel'
       write(irw,'(a)')' '
       write(irw,'(a)')'##'
       close(unit=irw)

       kk=0
       do j=2,ichanks(0),2 
        print '(2(a,i4),a)',' from ',ichanks(j-1)+1,' to ',ichanks(j),' step 2'
       do k=ichanks(j-1),ichanks(j),2
         kk=kk+1
         set1(kk)= real_fr(mfpt+k+1)
       end do
       end do
       irw=irw+1
       write(irw,'(a)')'even:'//TRIM(namedata)
       write(irw,'(a)')TRIM(format)//'    y'       
       write(irw,'(a,i5)')'nu_channel=',kk
       write(irw,format)(set1(k),k=1,kk)
       write(irw,'(a)')'counts'
       write(irw,'(a)')'channel'
       write(irw,'(a)')' '
       write(irw,'(a)')'##'
       close(unit=irw)



       
       deallocate(set1)      

       return      
      end if
    
      if(lq.eq.6)then
       print '(a,i2)',' data set number ',ks_set
       querytype='data first to be corrected by the geometry effect'
       ch1='y' 
       chcom='effi/folding/geometry' 
       call yesno(querytype,ch1,chcom,npl,nrd,nwr) 
       if(ch1.eq.'n')then
        call folding(0,datfilesp)
       else
        call folding(1,datfilesp)        
       end if
       return
      end if 

   end subroutine sum2datasets
      
    
                                                                        
      subroutine vadjust(icol,npanum,mf,length) 
      use comfrm, only: np,p,up,op,ddp
      implicit none                                                                        
      integer, intent(in) :: icol,mf,npanum(mf+2)
      integer, intent(out):: length

      character ch2*2 
      integer l,j,k,i,iadj,i0,ic1,ic2,ic4,lo,i1,ic0,i10 

      if(icol.eq.0) then 
       l=14 
       do j=1,npanum(2)
        k=npanum(j+2) 
        i=24 
        do while(np(k)(i:i).eq.' ') 
         i=i-1 
        end do 
        if(i.gt.l)    l=i 
       end do 
       length=l+1 
       return 
      end if 
                                                                        
      iadj=0 
      i0=ichar('0') 
      ic1=24+(icol-1)*9+5 
      ic2=ic1+1 
      ic0=ic1-1 
      ic4=ic2+2 
      length=3 
      k=npanum(3) 
      lo=10*(ichar(np(k)(ic1:ic1))-i0)+ichar(np(k)(ic2:ic2))-i0 
      do j=1,npanum(2) 
       k=npanum(j+2) 
       goto (10,20,30,40), icol 
   10  i10=abs(p(k)) 
       goto 50 
   20  i10=abs(up(k)) 
       goto 50 
   30  i10=abs(op(k)) 
       goto 50 
   40  i10=abs(ddp(k)) 
   50  continue 
       l=2 
       do while(i10.gt.1) 
        i10=i10/10 
        l=l+1 
       end do 
       if(np(k)(ic0:ic0).eq.'f') then 
        l=l+1+ichar(np(k)(ic4:ic4))-i0 
       end if 
       if(np(k)(ic0:ic0).eq.'e') then 
        l=6+ichar(np(k)(ic4:ic4))-i0 
       end if 
       if(l.gt.length) length=l 
       if(lo.eq.l) iadj=iadj+1 
      end do 
!       write(6,'(a,2i5)')'iadj,npanum(mscr1+2)=',                      
!     &                    iadj,npanum(mscr1+2)                         
!      if(iadj.ne.npanum(mscr1+2)) then                                 
       i10=length/10 
       i1=length-10*i10+i0 
       ch2=char(i0+i10)//char(i1) 
       do j=1,npanum(2) 
        k=npanum(j+2) 
        np(k)(ic1:ic2)=ch2 
       end do 
!      end if                                                           
      return 
    END subroutine vadjust


  real function PHI_inv(u)
!  Inversion for the normal distribution
!  Approximate PHI**-1(u) by AS70 of Odeh and Evans (1974).        
  real, intent(in):: u
  real(kind=8)   :: v,A,B
  real(kind=8), parameter:: a0=-0.322232431088,  b0= 0.099348626060,  &
                            a1=-1.0           ,  b1= 0.588581570495,  &
                            a2=-0.342242088547,  b2= 0.531103462366,  &
                            a3=-0.0204231210245, b3= 0.103537752850,  &
                            a4=-0.0000453642210148, b4= 0.0038560700634

 v =sqrt(-2.0d0*alog(u))
 A = (((a4* v + a3 )* v + a2 ) * v + a1 )*v + a0
 B = (((b4 *v + b3 )* v + b2 ) *v + b1 ) *v + b0

 Phi_inv= v + A/B
 return
end function PHI_inv

  subroutine mc_command(comline,iseed,info) 
     implicit none
     character(len=132), intent(inout):: comline
     integer, intent(out) :: info
     integer, intent(inout):: iseed
     character(len=80),dimension(8):: helpstring,querystring
     integer idimst,ist

     idimst=8
     ist=4

        helpstring(1)='effi/synthetic/errors' 
        helpstring(2)='effi/synthetic/mc_errors' 
!       PHI(-1)(r) from AS70 of Odeh and Evans (1974)
        helpstring(3)='effi/synthetic/complete/nonparalyzable' 
        helpstring(4)='effi/synthetic/complete/paralyzable' 
        querystring(1)='y=f+|y-f|*sign(r-0.5) (r=random number)' 
        querystring(2)='y=f+sqrt(f)*PHI**(-1)(r) (10**-5<r<1)' 
        querystring(3)='complete simulation (nonparalyzable)' 
        querystring(4)='complete simulation (paralyzable)' 
      call mccommand(comline,iseed,idimst,ist,helpstring,querystring,info)
     return
   end subroutine mc_command
   
     
      subroutine mcarlodata(lq,chft,mf,pathw,iprobe,info) 
        use glbconst, only: maxpf,maxc,max8,infotext,iseed,   &
                            infotext,enwr,nrd,nwr,ncl
      use comfrm, only: p,kp,pold,mp,np
      use comdat, only: mfpt,mdfpt,b_datset,nchandata,ichanks
      use comthe, only: real_fr,thfk,dfdp
      use comths, only: nfit,ntest,kf,iterations
      implicit none                                                                         
      
      integer, intent(in) :: lq,iprobe
      integer, intent(inout) :: mf
      integer, intent(out):: info
      real      pathw
      character(len=132):: ch132
      character(len=4)  :: chft,cmch
      character(len=14),dimension(3) :: formpr
      character(len=80) :: name
      character(len=8)  :: ch8
      character(len=2)  :: ch2
      character(len=1)  :: ch0

      real    RAND,PHI_inv
      integer,dimension(mf)   :: ifree,kpfr
      integer,dimension(mf+2) :: npanum
      integer,dimension(0:4)  :: length
      real,   dimension(iprobe)   :: chi_ar
      real,   dimension(mf)       :: apar,apar2
      real,   dimension(mf,iprobe):: par
      real,   dimension(:),allocatable:: thfklc
      
      logical  whichtheory,b_name
      integer  i0,kpa,mfout,i,i1,i10,ipl,iter_count,j,j1,k,       &
               ks,moca,nchansp
      real     chisqmin,ochisq,chisq,chi_p,path1,path2,chq,chq2,convcrit,dchi,&
               r,y,sq2

      allocate(thfklc(size(thfk)))
      thfklc=thfk
      
      ch0=char(0)                                                    
      if(chft.eq.' ') then 
       chft='newt' 
!       write(6,'(a)')'no choice of fit routine ([ne] inserted)'        
      end if 
      i0=ichar('0')

      mf=0
      do kpa=1,mp 
       if (kp(kpa).eq.1) then 
        mf=mf+1 
        ifree(mf)=kpa 
        kf(mf)=kpa 
        kpfr(mf)=kp(kpa) 
       end if 
      end do 
                                                                        
      if(lq.lt.3)then 
       if(mf.eq.0)then 
        info=1 
        infotext(1)='effi/command/mc/fit' 
        infotext(2)='**error** at least one free parameter for the fit' 
        infotext(3)=' ' 
        return 
       end if 
       npanum(2)=mf 
       do j=1,mf 
        npanum(j+2)=ifree(j) 
       end do 
                                                                        
       do k=0,4 
        call vadjust(k,npanum,mf,length(k)) 
       end do 
       i10=length(0)/10 
       i1=length(0)-i10*10+i0 
       ch2=char(i10+i0)//char(i1) 
                                                                        
       formpr(1)='2a1,i1,3x,a'//ch2//',' 
       formpr(2)='2a1,i2,2x,a'//ch2//',' 
       formpr(3)='2a1,i3,1x,a'//ch2//',' 
                                                                        
!       write(6,'(a,5i4)')'fitsub info,iprobe,iterations,nfit(0)=',     
!    &  mf,info,iprobe,iterations,nfit(0)                               
              ! lq.lt.3                                                 
      end if
    
      r=RAND(iseed)                                                                 
      chq2=0.0 
      chq =0.0 
      do moca=1,iprobe
       thfk=thfklc
         
       do k=1,nfit(0) 
         ks=nfit(k)
         if(b_datset(ks))then 
         j1=1 
         cmch='test'; call frametheory(ks,cmch,j1,info) 
         if(info.gt.0) goto 10 
         nchansp=nchandata(ks)
         info=1
         if(lq.eq.2) then 
          sq2=sqrt(2.0) 
          !----------------------------
          name='Mossbauer transmission'
          b_name=whichtheory(ks,name)
          name="Stroboscopy in transmission"
          b_name=b_name.or.whichtheory(ks,name)
          !----------------------------
          if(b_name) then  
           info=0
           do j=2,ichanks(0),2 
           do i=ichanks(j-1),ichanks(j) 
!           do i=1,nchansp 
            r=RAND(0) 
            
            if(r.lt.0.0001) r=0.0001
            y=sqrt(thfk(i))*PHI_inv(r)
            real_fr(mfpt+i)=int(thfk(i)+y) 
            real_fr(mdfpt+i)=sqrt(abs(real_fr(mfpt+i)))+0.1
           end do 
           end do
          end if
          !----------------------------
          name='Nuclear forward scattering'
          !----------------------------
          if(whichtheory(ks,name)) then 
           info=0
           do j=2,ichanks(0),2 
           do i=ichanks(j-1),ichanks(j) 
!           do i=1,nchansp 
            r=RAND(0) 
            if(r.lt.0.00001)r=0.00001
            y=sqrt(thfk(i))*PHI_inv(r)
            real_fr(mfpt+i)=int(thfk(i)+y) 
            real_fr(mdfpt+i)=sqrt(abs(real_fr(mfpt+i)))+1
           end do 
           end do
          end if
         else if(lq.eq.1)then
          info=0 
          do j=2,ichanks(0),2 
          do i=ichanks(j-1),ichanks(j) 
           y=real_fr(mfpt+i)-thfk(i) 
           if(RAND(0).gt.0.5) y=-y
           real_fr(mfpt+i)=int(thfk(i)+y) 
          end do 
          end do
         else if(lq.ge.3)then 
          info=0
!         true Monte Carlo ------------------------ 
          !----------------------------
          name='Mossbauer transmission'
          b_name=whichtheory(ks,name)
          name="Stroboscopy in transmission"
          b_name=b_name.or.whichtheory(ks,name)
          !----------------------------
          if(b_name) then 
           info=0 
           call counts_mcarlo(lq,iseed,nchansp,nrd,nwr) 
           do j=2,ichanks(0),2 
           do i=ichanks(j-1),ichanks(j) 
!           do i=1,nchansp 
            real_fr(mfpt+i)=int(dfdp(i)+0.5) 
            real_fr(mdfpt+i)=sqrt(abs(real_fr(mfpt+i))) 
           end do
           end do
           goto 10
          end if
         end if 
        end if 
        if(info.gt.0) then 
         infotext(1)='effi/simulation/montecarlo' 
         infotext(2)='no simulation' 
         infotext(3)=&
         'Monte Carlo routine not yet implemented for the current theory' 
         infotext(4)=' ' 
         goto 10  
        end if 
       end do

     

       do i=1,mf 
         kp(ifree(i))=kpfr(i) 
       end do 

       if(enwr(15).eq.'fold')then
          ch132=' '
          call folding(1,ch132)
       end if
!       write(6,'(a,10i4)')' fit mc kpfr=',(kpfr(i),i=1,mf)             
                                                                        
       if((mf.eq.0.or.mf.gt.maxpf).and.chft.ne.'test') then 
        info=1 
        infotext(1)='effi/mcarlo/fit/mf_0' 
        infotext(2)=                                                    &
          '**error** no free or number of fit parameter too large'      
        infotext(3)=                                                    &
          'the command {te} is executed if iterations is set to 1'      
        infotext(4)=' ' 
        goto 10 
       end if 
                                                                        
       if(iprobe.eq.1.or.iterations.le.1) then 
        chft='test' 
        do i=0,nfit(0) 
         ntest(i)=nfit(i) 
        end do 
       end if 
                                                                        
       path2=0.0 
       iter_count=1 
                                                                        
!        write(6,'(a,i4,3g15.5)')' iterations, pathw,convcrit=',  &      
!                                 iterations,pathw,convcrit            
       ochisq=1.0e10
       chisqmin=ochisq
       do while(iter_count.le.iterations) 
                                                                        
       call fit(chft,mf,mfout,iter_count,pathw,                             &
                      chisqmin,ochisq,chisq,chi_p,path1,path2,info)          
!        write(6,'(a,2i4,5g15.5)')'fit mc mf,iter_count,chi_sq,ochisq=', &
       !                                  mfout,iter_count,chisq,ochisq,path2
       
        if(path2.lt.1.e-10) path2=0.
        if(mf.gt.mfout) then
         info=10 
         infotext(1)='effi/mcarlo/fixp' 
         infotext(2)='only true fit parameter are accepted' 
         infotext(3)=' ' 
        end if 
        if(info.gt.0) goto 10 
                                                                        
        if(iter_count.gt.0) then 
         iter_count=iter_count+1 
         dchi=ochisq-chisq 
         if(dchi.gt.convcrit) then 
          ochisq=chisq 
         else 
          goto 5 
         end if 
        else 
         iter_count=-iter_count+1 
        end if 
       end do ! iter_count    
5      continue
       do i=1,mf
         par(i,moca)=p(kf(i))
       end do   
       chq=chq+ochisq
       chi_ar(moca)=ochisq 
       chq2=chq2+ ochisq**2
      end do  ! 1-iprobe 

      if(iprobe.eq.1.and.iterations.le.1)write(nwr,'(a)')               &
       ' chi_square value of generated spectrum:'                       
                                                                        
      if(iterations.gt.1.and.iprobe.gt.1) write(nwr,'(/,a)')            &
      ' Monte Carlo generated spectra are fitted with routine '//chft   
      if(iprobe.gt.1)then
      apar=0.   
       do i=1,mf
        do j=1,iprobe     
         apar(i)=apar(i)+ par(i,j)
        end do
       end do
      apar2=0.
      do i=1,mf
       do j=1,iprobe     
        apar2(i)=apar2(i)+ par(i,j)**2
       end do
      end do

      do i=1,mf
       write(nwr,'(a24,a2,e12.4,a,e12.4)')np(kf(i)),' =', apar(i)/iprobe,  &
            ' +-',sqrt(abs(apar2(i)-apar(i)**2/iprobe)/(iprobe-1))
      end do

      write(nwr,'(a)')' chi_square values of generated specta:' 
       ipl=ncl/8 
       i10=ipl/10 
       i1=ipl-i10*10+i0 
       ch2=char(i10+i0)//char(i1) 
       ch8='('//ch2//'f8.4)'
       write(nwr,ch8)(chi_ar(i),i=1,iprobe) 
       write(nwr,'(a,f10.4,a,f10.4)') '      <chi_square>=',chq/iprobe,  &
                        ' +-',sqrt(abs(chq2-chq**2/iprobe)/(iprobe-1))     
      else 
       if(iprobe.eq.1)write(nwr,'(a,f10.4)')'     chi_square=',chisq 
      end if 
   10 return 
      deallocate(thfklc)
    END subroutine mcarlodata
                                                                        
                                                                        
     subroutine counts_mcarlo(icase,iseed,nu_channel,nrd,nwr) 
     use comthe, only: thfk,dfdp
     implicit none 
     integer, intent(in) :: icase,iseed,nu_channel,nrd,nwr
                                                                       
     integer i,counts,ledit
     real    countrate,deadtime,frwindow,sweepfrequency,start,finish  
     character(len=132):: chcrd,chcom
     character(len=1)  :: ch1

     real,dimension(7):: value
     character(len=40),dimension(7):: name          

        do i=1,nu_channel 
         thfk(i)=thfk(i)/dfdp(i)
        end do 
        thfk(1)=1.
        
!       25:32 information about the helpfile (th110105)                 
!             if 25:32 is empty chcom is taken by subroutine conread    
!             as defined below                                          
!       33:37 format (effi specific) of the number                      
!                  1234567890123456789012345678901234567                
        name(1)='counts(v=oo)                    i08)' 
        name(2)='countrate(/sec)                 e08.1)' 
        name(3)='deadtime(sec)                   e08.2)' 
        name(4)='window_fraction                 f05.2)' 
        name(5)='sweepfrequency(Hz)              f05.1)'
        
        counts=100000         ! in the base line
        countrate=1.0e4       ! per sec in the detector (all counts)    
        deadtime=0.2e-4       ! countrate*deadtime=0.25 
        frwindow= 1.0         ! fraction of gated countrate (14.4 keV)
        sweepfrequency=0      ! : sweepfrequency ~ 1.0sec*countrate counts
                              ! per channel each sweep
        !sweepfrequency >0.0   1/sweepfrequency/nu_channel sec per channel
        
        
        chcom='<Mossbauer_spectrum>/trans/simulation/montecarlo' 
        value(1)=counts
        value(2)=countrate      ! in the detector
        value(3)=deadtime
        value(4)=frwindow 
        value(5)=sweepfrequency
        
        ledit=0         
        chcrd='1-5'          
        call conread(chcrd,value,name,5,chcom,ledit) 
                                                                       
        counts=        value(1) 
        countrate=     value(2) 
        deadtime=      value(3)
        frwindow=      value(4)
        sweepfrequency=value(5)
        
        chcrd='start of Monte Carlo ' 
        ch1='y' 
        chcom='effi/mcarlo_true' 
        call yesno(chcrd,ch1,chcom,80,nrd,nwr) 
        if(ch1.eq.'n') return
                                                                        
        call cpu_time(start) 
        call  mcarlocounts(icase,nu_channel,countrate,deadtime, &
                          sweepfrequency,counts,frwindow,iseed)  
                                                                         
        call cpu_time(finish) 
        print '(a,f15.3,a)',' runtime = ',finish-start,' seconds)'
                                                                                
        return 
      END subroutine  counts_mcarlo
                                                                        


    subroutine folding(ny,datfilesp) 
     use glbconst, only: nrd,nwr,ncl
     use comdat, only:  mfpt,mgpt,nchannelsp
     use comthe, only: real_fr     
     implicit none

     integer, intent(in):: ny
     real, dimension(-4:4):: pt,pt1,pt2
     character(len=132):: chcom,filename,querytype,datfilesp,ch132
     character(len=32) :: filetype,format
     character(len=16) :: exten
     character(len=1)  :: ch1

     integer l,i,k,j,js,ks,irw
     real    y,yx,yxx,a,b,c,h,ptmin,foldp,pm,fpb
     real, dimension(:), allocatable:: set1
       
    js=0; ks=0; ptmin=1.e20
    pt=0.; pt1=0.;  pt2=0.; pm=0.
    h=real_fr(mfpt+nchannelsp/2)
     do l=-4,4
      k=nchannelsp/2+l    
      i=k+1;j=k-1
      do while(i.gt.2.and.j.lt.nchannelsp) 
       i=i-1; j=j+1
       if(ny.eq.1)then 
        pt1(l)=pt1(l)+   &
        ((real_fr(mfpt+i)/real_fr(mgpt+i)-real_fr(mfpt+j)/real_fr(mgpt+j))/h)**2 
        pt2(l)=pt2(l)+   &
        ((real_fr(mfpt+i)/real_fr(mgpt+i)-real_fr(mfpt+j+1)/real_fr(mgpt+j+1))/h)**2
       else
         pt1(l)=pt1(l)+((real_fr(mfpt+i)-real_fr(mfpt+j))/h)**2   !+0
         pt2(l)=pt2(l)+((real_fr(mfpt+i)-real_fr(mfpt+j+1))/h)**2 !+ i+0.5           
       end if
      end do
       
      if(ptmin.gt.pt1(l))then
        ptmin=pt1(l)
!        print'(i6,2x,g10.4)',k,ptmin
        foldp=k
      end if
      if(ptmin.gt.pt2(l))then
        ptmin=pt2(l)
!        print'(f6.1,2x,g10.4)',k+0.5,ptmin
        foldp=k+0.5
      end if   
     end do !l=-4,4

     pt=pt1
     if(int(2*foldp).gt.2*int(foldp))pt=pt2

     y=0.; yx=0.; yxx=0.
     do l=-4,4
      y=y+pt(l)
      yx=yx+pt(l)*float(l)
      yxx=yxx+pt(l)*float(l*l)
     end do
     b=yx/60.
     a=(yxx/2.-y*10./3.)/154.
     c= (y/3.-20.*a)/3.
    
      fpb=-b/2./a
    if(a.lt.0.) then
       print'(a,/,a)',' all vertices of the parabola are maxima',&
            ' no standard folding point located'
      foldp= nchannelsp
    else
     print'(a,f10.3)',' optimal folding point =',fpb+float(nchannelsp/2)
     print'(a,f10.3)',' standard folding point=', foldp
    end if   

    querytype='folding the spectrum and save to file'
    ch1='n' 
    chcom='effi/folding/foldspectrum' 
    call yesno(querytype,ch1,chcom,ncl,nrd,nwr) 
    if(ch1.eq.'n') goto 100
    irw=15
    
    allocate(set1(2048))
    
    k=int(foldp);
    if(float(k)-foldp.lt.0) then
      i=k+1;j=k
    else
     i=k+1;j=k-1
    end if
    l=0
    do while(i.gt.2.and.j.lt.nchannelsp)
     i=i-1;j=j+1  
      set1(i)= real_fr(mfpt+i)+ real_fr(mfpt+j)
     l=l+1
    end do
    ! 1- i-1 and j+1-nchannelsp
    if(i-1.gt.1)then
     do k=1,i-1   
        set1(k)= real_fr(mfpt+k)+real_fr(mfpt+nchannelsp-k)
     end do
    end if
    if(j.lt.nchannelsp)then
     do k=j+1,nchannelsp   
        set1(k)= real_fr(mfpt+k)+real_fr(mfpt+nchannelsp-k) 
     end do
    end if
  
    chcom='effi/contraction_sp' 
    filetype='counts/channel' 
    exten='dat'
    j=index(datfilesp,'.')-1
    filename=datfilesp(:j)//'_fold.dat'
    call rdfname(filetype,filename,exten,chcom,ncl,nwr)
      
    open(unit=irw,file=filename                     &
                ,status='unknown'                  &
                ,access='sequential'               &
                ,form='formatted')                  


       querytype=' the format of the counts'
       ch132='(5x,f10.0)'
       chcom='effi/counts/format' 
       call askstring(querytype,ch132,chcom,ncl,nwr)       
       format=ch132 
       write(irw,'(a,f10.3)')' standard folding point=', foldp
       write(irw,'(a)')TRIM(format)//'    y'       
       write(irw,'(a,i5)')'nu_channel=',nchannelsp/2
       write(irw,format)(set1(k),k=1,nchannelsp/2)
       write(irw,'(a)')'counts'
       write(irw,'(a)')'channel'
       write(irw,'(a)')' '
       write(irw,'(a)')'##'
       close(unit=irw)
       deallocate(set1)      

100    return
   end subroutine folding
   
