      subroutine lineq(ncase,nd,u,dp) 
      implicit none 
      integer ncase,nd,                                  &
              i,j,l,k,kk,k1,ll,ii,jj,j1 
      real,dimension(nd,nd):: u
      real,dimension(nd) :: dp 
      real sum 
                                                                        
!     ******************************************************************
!     solving systems of linear equations by the method of cholesky     
!     calculation of inverse matrix                                     
!     ******************************************************************
!     ncase = 0   linear equations, u = left sides, dp = right sides    
!     solution stored in dp                                             
!     ncase = 1   calculation of the inverse matrix of u                
!     stored in u                                                       
!     u = symmetrical nd*nd-matrix, only upper triangle is required     
!     dp = vector(nd), dummy for matrix inversion                       
!     ******************************************************************
!                                                                       
!     choleski                                                          
!                                                                       
      do k=1,nd 
       kk=k-1 
       if (kk.gt.0) then 
        do k1=1,kk 
         if (ncase.eq.0) dp(k)=dp(k)-u(k1,k)*dp(k1) 
         do l=k,nd 
          u(k,l)=u(k,l)-u(k1,l)*u(k1,k) 
         end do 
        end do 
       end if 
                                                                        
       if (u(k,k).gt.0) then 
        u(k,k)=sqrt(u(k,k)) 
       else if(u(k,k).lt.0) then 
        u(k,k)=sqrt(-u(k,k)) 
       end if 
       if (u(k,k).lt.1.e-4) u(k,k)=1.e-4 
                                                                        
       if (ncase.eq.0) dp(k)=dp(k)/u(k,k) 
                                                                        
       kk=k+1 
       if (nd-kk.lt.0) goto 60 
       do l=kk,nd 
        u(k,l)=u(k,l)/u(k,k) 
       end do 
      end do 
!                                                                       
!     back substitution                                                 
!                                                                       
   60 if (ncase.eq.1) goto 100 
                                                                        
      do k=1,nd 
       kk=nd-k+1 
       if (k-1.ne.0)then 
        ll=kk+1 
        do l=ll,nd 
         dp(kk)=dp(kk)-u(kk,l)*dp(l) 
        end do 
       end if 
       dp(kk)=dp(kk)/u(kk,kk) 
      end do 
                                                                        
      goto 160 
!                                                                       
!     calculation of the inverse matrix                                 
!                                                                       
  100 do kk=1,nd 
       k=nd-kk+1 
       do ii=kk,nd 
        i=nd-ii+1 
        jj=i+1 
        sum =0. 
        if(i-nd.eq.0) goto 115 
        do j=jj,nd 
         j1=j 
         k1=k 
         if(k-j.lt.0) then 
          k1=j 
          j1=k 
         end if 
         sum=sum-u(i,j)*u(i,i)*u(k1,j1) 
        end do 
  115   if (k.eq.i) sum=sum+1 
        u(k,i)=sum/u(i,i)**2 
       end do 
      end do 
                                                                        
      do  k=1,nd 
       do  j=k+1,nd 
        u(k,j)=u(j,k) 
       end do 
      end do 
  160 return 
      END                                           
                                                                        
      function FUNC(pp,mf) 
      use comfrm, only: p,op,up
      use comdat, only: mfpt,mdfpt,b_datset,ichan
      use comthe, only: real_fr,thfk
      use comths, only: kf,i_opgtup,nfit
      implicit none 
      integer, intent(in):: mf 
      real FUNC,z 
      real, dimension(mf):: pp 
      character(len=4):: cmch
                                                                  
      integer i,kpa,info,j,j1,k,ks,nf 

      do i=1,mf 
       kpa=kf(i) 
       if(i_opgtup(i).eq.1) then 
        if(pp(i).gt.op(kpa))then 
         pp(i)=op(kpa) 
        else if(pp(i).lt.up(kpa))then 
         pp(i)=up(kpa) 
        end if 
       end if 
       p(kpa)=pp(i) 
      end do
      call calpl('free') 
                                                                        
!     parameter p defined                                               
       info=0 
       z=0. 
       nf=0 
       do k=1,nfit(0) 
        ks=nfit(k) 
        if(b_datset(ks))then 
         j1=1 
         cmch='test'; call frametheory(ks,cmch,j1,info) 
         if(info.gt.0) goto 10 
         do j=2,ichan(0,ks),2 
          do i=ichan(j-1,ks),ichan(j,ks) 
           nf=nf+1 
           z=z+((real_fr(mfpt+i)-thfk(i))/real_fr(mdfpt+i))**2 
          end do 
         end do 
        end if 
       end do 
       if(nf.gt.0) then 
        FUNC=z/(nf-mf)
        return 
       end if 
   10  FUNC=0. 
       return 
      END                                           
                                                                        
      subroutine DFUNC(pp,dpp,f0,df,mf) 
      implicit none 
      integer mf,i
      real fd,f0,FUNC
      real,dimension(mf)::pp,dpp,df 
                                                                      
       f0=FUNC(pp,mf) 
       do i=1,mf 
        pp(i)=pp(i)+dpp(i) 
        fd=FUNC(pp,mf)
        pp(i)=pp(i)-dpp(i) 
        df(i)=(fd-f0)/dpp(i) 
       end do 
      return 
      END                                           
                                                                        
      subroutine DDFUNC(pp,dpp,ddf,mf) 
      implicit none 
      integer mf,i,k 
      real,dimension(mf)   :: pp,dpp
      real,dimension(mf,mf):: ddf                                                                        
      real FUNC,f0,fdi,fdk,fdii,fdik 
                                                                        
      f0=FUNC(pp,mf) 
!      write(6,'(a,g15.5,i4)')' DDFUNC f0,nf=',f0,nf                    
      do i=1,mf 
       pp(i)=pp(i)+dpp(i) 
       fdi=FUNC(pp,mf) 
       pp(i)=pp(i)+dpp(i) 
       fdii=FUNC(pp,mf) 
       pp(i)=pp(i)-2.0*dpp(i) 
       ddf(i,i)=(fdii-2.0*fdi+f0)/dpp(i)**2 
!      write(6,'(2i4,a,6g12.4)')i,i,' DDFUNC ddf=',ddf(i,i)             
       do k=i+1,mf 
        pp(k)=pp(k)+dpp(k) 
        fdk=FUNC(pp,mf) 
        pp(i)=pp(i)+dpp(i) 
        fdik=FUNC(pp,mf) 
        pp(k)=pp(k)-dpp(k) 
        pp(i)=pp(i)-dpp(i) 
        ddf(i,k)=(fdik-fdi-fdk+f0)/dpp(i)/dpp(k) 
        ddf(k,i)=ddf(i,k) 
!       write(6,'(2i4,a,6g12.4)')i,k,' DDFUNC ddf=',ddf(i,k)            
       end do 
      end do 
      return 
      END                                           
                                                                         
                                                                        
       SUBROUTINE POWELL(nametty,PP,dpp,XX,N,iter_count,           &
                        FTOL,ITERBRAK,iBR_DBR,ibx_calc,      &
                        kf,namepp,pplast,nwr,FRET)
      use glbconst, only: quit
      implicit none 
      character(len=12),intent(in):: nametty

      real,  intent(in):: FTOL 
      real,dimension(N), intent(in)  :: dpp
      integer, intent(in):: N,iter_count,ITERBRAK,iBR_DBR,ibx_calc,nwr 
      real,   dimension(N),intent(inout):: PP
      integer,dimension(N), intent(in):: kf 
      character(len=24),dimension(N), intent(in):: namepp 
      real,   dimension(N,N),intent(out):: XX                              
      real,   intent(out):: FRET 

      integer,dimension(N):: ITER_M 
      real,dimension(N)  :: PT,PTT,XIT,df,pplast,PPmin
      integer   I,J,ITMAX,IBIG,ITERM,ITER 
      real      FUNC,DEL,FP,FPTT,T,FPmin
                                                                        
      PTT=0.0
      XX=0.0
      df=0.0
      do I=1,N 
       XX(I,I)=dpp(I) 
      end do 
                                                                        
      FRET=FUNC(PP,N) 
      PT=PP
!     input iter is maximal iterations; output actual iterations        
      ITMAX=iter_count 
      ITER=0
      FPmin=1.0e10
1     ITER=ITER+1
      IF(FPmin.gt.FRET)then
         FPmin=FRET
         PPmin=PP
      end IF   
      FP=FRET 
      IBIG=0 
      DEL=0. 
      ITERM=0 
      write(nwr,'(i4,a)')ITER,'. iteration: Powell' 
        write(nwr,'(2x,a2,5x,a4,24x,a,10x,a,10x,a)')                    &
                     'nu','name','value','d-value','int. loops'        

        call  askquit(nametty)
        if(quit.eq.'q')then
         PP=PPmin  
         return
        end if
                                                                        
      DO I=1,N 
        DO J=1,N 
          XIT(J)=XX(J,I) 
        END DO
        CALL LINMIN(PP,dpp,PTT,df,XIT,N,FRET,                           &
                    FTOL,iBR_DBR,ibx_calc,ITERBRAK,ITERM) 
        ITER_M(I)=ITERM 
        IF(ABS(FP-FRET).GT.DEL)THEN 
          DEL=ABS(FP-FRET) 
          IBIG=I 
        ENDIF 
      END DO 
                                                                        
      do I=1,N 
       write(nwr,'(x,a1,i3,a1,2x,a24,g15.5,2x,g15.5,8x,i3)')            &
         '(',kf(I),')',namepp(I),PP(I),PP(I)-pplast(I), ITER_M(I)      
      end do 
      write(nwr,'(a,g15.5)') ' chi**2=',FRET 
                                                                        
                                                                        
      IF(2.0*ABS(FP-FRET).LE.FTOL*(ABS(FP)+ABS(FRET)))then 
       write(nwr,'(a,f12.5,a)')                                         &
       'local minimum  (|d chi**2|/chi**2 <',FTOL,') reached'          
       PP=PPmin
       RETURN 
      end if 
      IF(ITER.EQ.ITMAX) then 
       write(nwr,'(a)')' termination without convergence' 
       PP=PPmin
       RETURN 
      end if 

   

      DO J=1,N 
        PTT(J)=2.0*PP(J)-PT(J) 
        XIT(J)=PP(J)-PT(J) 
        PT(J)=PP(J) 
      end do
      
      FPTT=FUNC(PTT,N) 
                                                                        
      T=2.0*(FP-2.0*FRET+FPTT)*(FP-FRET-DEL)**2-DEL*(FP-FPTT)**2 
!print'(a,5f12.5)','FPTT,FP,T=',FPTT,FP,T
      IF(FPTT.GE.FP.or.T.GE.0.0) GOTO 1 
!print'(a,5f12.5)','       PP=',PP                                                                        
       CALL LINMIN(PP,dpp,PTT,df,XIT,N,FRET,                            &
            FTOL,iBR_DBR,ibx_calc,ITERBRAK,ITERM)
!print'(a,5f12.5)','       PP=',PP                                                                        
!print'(a,5f12.5)','      PTT=',PTT                                                                        
      write(nwr,'(a,g15.5)') ' chi**2=',FRET 
                                                                        
      DO J=1,N 
        XX(J,IBIG)=XIT(J) 
      end do 
                                                                        
      GO TO 1 
      END                                           
                                                                        
      SUBROUTINE LINMIN(PP,dpp,PTT,df,XI,N,FRET,                        &
                        FTOL,iBR_DBR,ibx_calc,ITERBRAK,ITERM)           
      implicit none 
      integer  N,ITERBRAK,iBR_DBR,ibx_calc,ITERM 
      real     FTOL,FRET 
      real, dimension(N):: PP,dpp,PTT,df,XI 
      real     ax,bx,cx,XMIN,DBRENT,BRENT 
      integer  ITER,J

      XMIN=0. 
      ax=0.0 
      bx=1.0
      if(ibx_calc.eq.1) then 
       do J=1,N 
!        if(abs(bx*XI(J)).gt.dpp(J)) bx=abs(dpp(J)/XI(J)) 
        if(abs(XI(J)).gt.dpp(J)) bx=abs(dpp(J)/XI(J)) 
       end do 
      end if 
      cx=2.0*bx 
      ITER=ITERBRAK

      CALL MNBRAK(PP,PTT,XI,N,ax,bx,cx,ITER) 
      if(iBR_DBR.eq.1) then 
       FRET=BRENT(PP,PTT,XI,N,ax,bx,cx,FTOL,XMIN,ITER) 
      else 
       FRET=DBRENT(PP,dpp,PTT,df,XI,N,ax,bx,cx,FTOL,XMIN,ITER) 
      end if 
      if(ITERM.lt.ITER)ITERM=ITER 
      XI=XMIN*XI
      PP=PP+XI
!      DO J=1,N 
!        XI(J)=XMIN*XI(J) 
!        PP(J)=PP(J)+XI(J) 
!      END DO 
      RETURN 
      END                                           
                                                                        
      SUBROUTINE MNBRAK(PP,PTT,XI,N,AX,BX,CX,ITER) 
      implicit none 
      real     AX,BX,CX 
      integer  N,ITER,ITMAX 
      real,dimension(N):: PP,PTT,XI 
      real,PARAMETER :: GOLD=1.618034, GLIMIT=10.0, TINY=1.0e-6 
      REAL      U,FA,FB,FC,FU,F1DIM,DUM,R,Q,ULIM 
                                                                        
                                                                        
!      write(6,'(a,6g15.6)')' mnbrak XI:',XI                            
!      write(6,'(a,6g15.6)')' mnbrak PP:',PP                            
      FA=F1DIM(PP,PTT,XI,N,AX) 
!      write(6,'(a,6g15.6)')' mnbrak AX,PTT:',AX,PTT                    
      FB=F1DIM(PP,PTT,XI,N,BX) 
!      write(6,'(a,6g15.6)')' mnbrak BX,PTT:',BX,PTT                    
                                                                        
      IF(FB.GT.FA)THEN 
        DUM=AX 
        AX=BX 
        BX=DUM 
        DUM=FB 
        FB=FA 
        FA=DUM 
      ENDIF 
      CX=BX+GOLD*(BX-AX) 
      FC=F1DIM(PP,PTT,XI,N,CX) 
!      write(6,'(a,6g15.6)')' mnbrak :',AX,BX,CX,FA,FB,FC               
      ITMAX=ITER 
      ITER=-1 
    1 IF(FB.GE.FC)THEN 
        ITER=ITER+1 
!        write(6,'(a,2i4)')'MNBRAK ITMAX,iter=',ITMAX,ITER              
        if(ITER.gt.ITMAX) goto 2 
        R=(BX-AX)*(FB-FC) 
        Q=(BX-CX)*(FB-FA) 
        U=BX-((BX-CX)*Q-(BX-AX)*R)/                                     &
                      (2.0*SIGN(MAX(ABS(Q-R),TINY),Q-R))                
        ULIM=BX+GLIMIT*(CX-BX) 
        IF((BX-U)*(U-CX).GT.0.0)THEN 
          FU=F1DIM(PP,PTT,XI,N,U) 
!          write(6,'(a,5g15.5)')'1MNBRAK U=',U,FU                       
          IF(FU.LT.FC)THEN 
            AX=BX 
            FA=FB 
            BX=U 
            FB=FU 
            GO TO 1 
          ELSE IF(FU.GT.FB)THEN 
            CX=U 
            FC=FU 
            GO TO 1 
          ENDIF 
          U=CX+GOLD*(CX-BX) 
          FU=F1DIM(PP,PTT,XI,N,U) 
!          write(6,'(a,5g15.5)')'2MNBRAK U=',U,FU                       
        ELSE IF((CX-U)*(U-ULIM).GT.0.0)THEN 
          FU=F1DIM(PP,PTT,XI,N,U) 
!          write(6,'(a,5g15.5)')'3MNBRAK U=',U,FU                       
          IF(FU.LT.FC)THEN 
            BX=CX 
            CX=U 
            U=CX+GOLD*(CX-BX) 
            FB=FC 
            FC=FU 
            FU=F1DIM(PP,PTT,XI,N,U) 
          ENDIF 
        ELSE IF((U-ULIM)*(ULIM-CX).GE.0.0)THEN 
          U=ULIM 
          FU=F1DIM(PP,PTT,XI,N,U) 
!          write(6,'(a,5g15.5)')'4MNBRAK U=',U,FU                       
        ELSE 
          U=CX+GOLD*(CX-BX) 
          FU=F1DIM(PP,PTT,XI,N,U) 
!          write(6,'(a,5g15.5)')'5MNBRAK U=',U,FU                       
        ENDIF 
        AX=BX 
        BX=CX 
        CX=U 
        FA=FB 
        FB=FC 
        FC=FU 
        if(abs(FA-FB)+abs(FB-FC).lt.TINY) then 
         ITER=1 
         goto 2 
        end if 
        GO TO 1 
      ENDIF 
      ITER=ITMAX 
    2 RETURN 
      END                                           
                                                                        
      FUNCTION BRENT(PP,PTT,XI,N,AX,BX,CX,TOL,XMIN,ITER) 
      implicit none 
                                                                        
      real     BRENT,AX,BX,CX,TOL,XMIN
      integer  N,ITER,ITMAX 
      real,dimension(N):: PP,PTT,XI 
      REAL     U,FU,F1DIM,R,Q,TOL1,TOL2,                                &
               A,B,V,W,X,E,FX,FV,FW,XM,P,ETEMP,D                            
                                                                        
      real, PARAMETER:: CGOLD=.381966,TINY=1.0e-6 
                                                                        
      A=MIN(AX,CX) 
      B=MAX(AX,CX) 
      V=BX 
      W=V 
      X=V 
      E=0.0 
      FX=F1DIM(PP,PTT,XI,N,X) 
      FV=FX 
      FW=FX 
      ITMAX=ITER 
!      write(6,'(a,6g15.6)')' mnbrak :',BX,FX                           
      DO ITER=1,ITMAX 
!        write(6,'(a,2i4)')'BRENT ITMAX,iter=',ITMAX,ITER               
                                                                        
        XM=0.5*(A+B) 
        TOL1=TOL*ABS(X)+TINY 
        TOL2=2.0*TOL1 
        IF(ABS(X-XM).LE.(TOL2-0.5*(B-A))) GOTO 3 
        IF(ABS(E).GT.TOL1) THEN 
          R=(X-W)*(FX-FV) 
          Q=(X-V)*(FX-FW) 
          P=(X-V)*Q-(X-W)*R 
          Q=2.0*(Q-R) 
          IF(Q.GT.0.0) P=-P 
          Q=ABS(Q) 
          ETEMP=E 
          E=D 
          IF(ABS(P).GE.ABS(0.5*Q*ETEMP).OR.P.LE.Q*(A-X).OR.             &
              P.GE.Q*(B-X)) GOTO 1                                      
          D=P/Q 
          U=X+D 
          IF(U-A.LT.TOL2 .OR. B-U.LT.TOL2) D=SIGN(TOL1,XM-X) 
          GOTO 2 
        ENDIF 
    1   IF(X.GE.XM) THEN 
          E=A-X 
        ELSE 
          E=B-X 
        ENDIF 
        D=CGOLD*E 
    2   IF(ABS(D).GE.TOL1) THEN 
          U=X+D 
        ELSE 
          U=X+SIGN(TOL1,D) 
        ENDIF 
        FU=F1DIM(PP,PTT,XI,N,U) 
!        write(6,'(a,2F10.3)')'BRENT FU=',U,FU                          
        IF(FU.LE.FX) THEN 
          IF(U.GE.X) THEN 
            A=X 
          ELSE 
            B=X 
          ENDIF 
          V=W 
          FV=FW 
          W=X 
          FW=FX 
          X=U 
          FX=FU 
        ELSE 
          IF(U.LT.X) THEN 
            A=U 
          ELSE 
            B=U 
          ENDIF 
          IF(FU.LE.FW .OR. W.EQ.X) THEN 
            V=W 
            FV=FW 
            W=U 
            FW=FU 
          ELSE IF(FU.LE.FV .OR. V.EQ.X .OR. V.EQ.W) THEN 
            V=U 
            FV=FU 
          ENDIF 
        ENDIF 
      END DO 
!      write(6,'(a)') 'Brent exceed maximum iterations.'                
    3 XMIN=X 
      BRENT=FX 
      RETURN 
      END                                           
                                                                        
                                                                        
      FUNCTION F1DIM(PP,PTT,XI,N,X) 
      implicit none 
      integer N 
      real F1DIM,X,FUNC 
      real,DIMENSION(N):: PP,PTT,XI 
       
      PTT=PP+X*XI                                                                 
!      DO J=1,N 
!        PTT(J)=PP(J)+X*XI(J) 
!      END DO 
      F1DIM=FUNC(PTT,N) 
      RETURN 
      END                                           
                                                                        
      SUBROUTINE DFPMIN(PP,dpp,HESSIN,N,ITER,                           &
                        FTOL,ITERBRAK,iBR_DBR,ibx_calc,                 &
                        df,kf,namepp,pplast,nwr,FRET)
      implicit none 
                                                                        
      character(len=24),dimension(1:N):: namepp 
      real    FTOL,FRET 
      Integer N,ITER,ITERBRAK,iBR_DBR,ibx_calc,nwr 
      real,dimension(N,N) :: HESSIN                             
      real,dimension(N)   :: PP,dpp,df,DG,HDG,XIT,pplast
      integer,dimension(N):: kf 
                                                                        
      integer I,J,ITS,ITMAX,ITERM 
      real    FP,FAD,FAE,FAC 
       real, PARAMETER::TINY=1.E-6 
                                                                        
      HESSIN=0.0 
      do I=1,N 
       HESSIN(I,I)=1.0 
      end do 
                                                                        
                                                                        
      call DFUNC(PP,dpp,FP,df,N) 

      XIT=-df 
      DG=df 

      ITMAX=ITER 
      ITER=0 
      DO  ITS=1,ITMAX 
      ITERM=0 
                                                                        
      ITER=ITS 
      write(nwr,'(i4,a)')ITER,'. iteration: D.F.P.' 
        write(nwr,'(2x,a2,5x,a4,24x,a,12x,a)')                          &
                     'nu','name','value','d-value'                     
                                                                        
        CALL LINMIN(PP,dpp,HDG,df,XIT,N,FRET,                           &
                    FTOL,iBR_DBR,ibx_calc,ITERBRAK,ITERM) 
                                                                        
        do I=1,N 
         write(nwr,'(x,a1,i3,a1,2x,a24,g15.5,2x,g15.5)')                &
         '(',kf(I),')',namepp(I),PP(I),PP(I)-pplast(I)                 
        end do 
        write(nwr,'(a,g15.5,a,i3)') ' chi**2=',FRET,                    &
                                   ' internal loops=',ITERM            
                                                                        
        IF(2.*ABS(FRET-FP).LE.FTOL*(ABS(FRET)+ABS(FP)+TINY)) then 
         write(nwr,'(a,f12.5,a)')                                       &
       'local minimum  (|d chi**2|/chi**2 <',FTOL,') reached'          
         RETURN 
        end if 
        IF(ITER.EQ.ITMAX) then 
         write(nwr,'(a)')' termination without convergence' 
         RETURN 
        end if 
        FP=FRET 
        CALL DFUNC(PP,dpp,FRET,df,N) 
        DG=df-DG 
!        write(6,'(a,6g12.3)')' DFPMIN DG=',DG                          
        DO I=1,N 
          HDG(I)=0. 
          DO J=1,N 
            HDG(I)=HDG(I)+HESSIN(I,J)*DG(J) 
          end do 
        end do 
!        write(6,'(a,6g12.3)')' DFPMIN HESSIN=',HESSIN                  
!        write(6,'(a,6g12.3)')' DFPMIN HDG   =',HDG                     
                                                                        
        FAC=0. 
        FAE=0. 
        DO I=1,N 
          FAC=FAC+DG(I)*XIT(I) 
          FAE=FAE+DG(I)*HDG(I) 
        end do 
        FAC=1./FAC 
        FAD=1./FAE 
        DO I=1,N 
          DG(I)=FAC*XIT(I)-FAD*HDG(I) 
        end do 
!        write(6,'(a,6g12.3)')' DFPMIN new DG=',DG                      
                                                                        
        DO I=1,N 
          DO J=1,N 
            HESSIN(I,J)=HESSIN(I,J)+FAC*XIT(I)*XIT(J)                   &
                       -FAD*HDG(I)*HDG(J)+FAE*DG(I)*DG(J)                        
          end do 
        end do 
                                                                        
        DO I=1,N 
          XIT(I)=0. 
          DO J=1,N 
            XIT(I)=XIT(I)-HESSIN(I,J)*df(J) 
          end do 
        end do 
!        write(6,'(a,6g12.3)')' DFPMIN newHESSIN=',HESSIN               
!        write(6,'(a,6g12.3)')' DFPMIN newXIT   =',XIT                  
      END DO 
      RETURN 
      END                                           
                                                                        
      FUNCTION DF1DIM(PP,dpp,PTT,df,XI,N,X) 
      implicit none 
      integer N,J 
      real DF1DIM,X,F0 
      real,DIMENSION(N):: PP,dpp,PTT,XI,df 
                                                                        
      DO J=1,N 
        PTT(J)=PP(J)+X*XI(J) 
      END DO 
      CALL DFUNC(PTT,dpp,F0,df,N)
      DF1DIM=0. 
      DO J=1,N 
        DF1DIM=DF1DIM+df(J)*XI(J) 
      END DO 
      RETURN 
      END                                           
                                                                        
                                                                        
                                                                        
      FUNCTION DBRENT(PP,dpp,PTT,df,XI,N,AX,BX,CX,TOL,XMIN,ITER) 
      implicit none 
                                                                        
      real DBRENT,AX,BX,CX,TOL,XMIN 
      integer N,ITER,ITMAX 
      real,dimension(N):: PP,dpp,PTT,df,XI 
      REAL U,FU,F1DIM,DF1DIM,TOL1,TOL2,OLDE,                            &
           A,B,V,W,X,E,FX,FV,FW,XM,D,DX,DU,DV,DW,D1,D2,U1,U2            
      real,PARAMETER :: TINY=1.0E-6 
      LOGICAL OK1,OK2 
                                                                        
      A=MIN(AX,CX) 
      B=MAX(AX,CX) 
      V=BX 
      W=V 
      X=V 
      E=0. 
      FX=F1DIM(PP,PTT,XI,N,X) 
      FV=FX 
      FW=FX 
      DX=DF1DIM(PP,dpp,PTT,df,XI,N,X) 
      DV=DX 
      DW=DX 
      ITMAX=ITER 
!      write(6,'(a,6g15.6)')' DBRENT    df:',df                         
                                                                        
      DO ITER=1,ITMAX 
        XM=0.5*(A+B) 
        TOL1=TOL*ABS(X)+TINY 
        TOL2=2.*TOL1 
!        write(6,'(a,4g12.6)')'DBRENT A,B,TOL1=',A,B,TOL*ABS(X)         
        IF(ABS(X-XM).LE.(TOL2-.5*(B-A))) GOTO 3 
        IF(ABS(E).GT.TOL1) THEN 
          D1=2.*(B-A) 
          D2=D1 
          IF(DW.NE.DX) D1=(W-X)*DX/(DX-DW) 
          IF(DV.NE.DX) D2=(V-X)*DX/(DX-DV) 
          U1=X+D1 
          U2=X+D2 
          OK1=((A-U1)*(U1-B).GT.0.).AND.(DX*D1.LE.0.) 
          OK2=((A-U2)*(U2-B).GT.0.).AND.(DX*D2.LE.0.) 
          OLDE=E 
          E=D 
          IF(.NOT.(OK1.OR.OK2))THEN 
            GO TO 1 
          ELSE IF (OK1.AND.OK2)THEN 
            IF(ABS(D1).LT.ABS(D2))THEN 
              D=D1 
            ELSE 
              D=D2 
            ENDIF 
          ELSE IF (OK1)THEN 
            D=D1 
          ELSE 
            D=D2 
          ENDIF 
          IF(ABS(D).GT.ABS(0.5*OLDE))GO TO 1 
          U=X+D 
          IF(U-A.LT.TOL2 .OR. B-U.LT.TOL2) D=SIGN(TOL1,XM-X) 
          GOTO 2 
        ENDIF 
    1   IF(DX.GE.0.) THEN 
          E=A-X 
        ELSE 
          E=B-X 
        ENDIF 
        D=0.5*E 
    2   IF(ABS(D).GE.TOL1) THEN 
          U=X+D 
          FU=F1DIM(PP,PTT,XI,N,U) 
        ELSE 
          U=X+SIGN(TOL1,D) 
          FU=F1DIM(PP,PTT,XI,N,U) 
          IF(FU.GT.FX)GO TO 3 
        ENDIF 
        DU=DF1DIM(PP,dpp,PTT,df,XI,N,U) 
        IF(FU.LE.FX) THEN 
          IF(U.GE.X) THEN 
            A=X 
          ELSE 
            B=X 
          ENDIF 
          V=W 
          FV=FW 
          DV=DW 
          W=X 
          FW=FX 
          DW=DX 
          X=U 
          FX=FU 
          DX=DU 
        ELSE 
          IF(U.LT.X) THEN 
            A=U 
          ELSE 
            B=U 
          ENDIF 
          IF(FU.LE.FW .OR. W.EQ.X) THEN 
            V=W 
            FV=FW 
            DV=DW 
            W=U 
            FW=FU 
            DW=DU 
          ELSE IF(FU.LE.FV .OR. V.EQ.X .OR. V.EQ.W) THEN 
            V=U 
            FV=FU 
            DV=DU 
          ENDIF 
        ENDIF 
      END DO 
!      PAUSE 'DBRENT exceeded maximum iterations.'                      
    3 XMIN=X 
      DBRENT=FX 
!      write(6,'(a,6g15.6)')' DBRENT  XMIN,FX:',XMIN,FX                 
      RETURN 
      END                                           
                                                                        
                                                                        
